/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import borland.jbcl.util.Variant;
import borland.jbcl.dataset.*;

public class JdbComboBox extends JComboBox
  implements NavigationListener, AccessListener, ItemListener,
             ColumnAware, java.io.Serializable
{
  public JdbComboBox() {
    super();
  }

  public JdbComboBox(ComboBoxModel aModel) {
    super(aModel);
  }

  /**
   * The items property fills the popuplist with a list of items
   * to pick from.
   */
  public synchronized void setItems(String[] items) {
    removeAllItems();
	if (items != null)
      for (int i = 0; i < items.length; i++)
        addItem(items[i]);
  }
  public synchronized void setItems(Object[] items) {
    removeAllItems();
    if (items != null)
      for (int i = 0; i < items.length; i++)
        addItem(items[i]);
  }
  public synchronized String[] getItems() {
    String[] items = new String[getModel().getSize()];
    for (int i = 0; i < getModel().getSize(); i++) {
      Object data = getModel().getElementAt(i);
      items[i] = data != null ? data.toString() : "";
    }
    return items;
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to drive the list's selection.
   */
  public void setDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    openDataSet(newDataSet);
    repaint();
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  /**
   * The columnName property specifies a column name in the dataSet
   * object to drive the list's selection.
   */
  public void setColumnName(String columnName) {
    this.columnName = columnName;
    openDataSet(dataSet);
    repaint();
  }
  public String getColumnName() {
    return columnName;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet(dataSet);
      }
    }
  }

  private void openDataSet(DataSet newDataSet) {
    if (!addNotifyCalled) {
      return;
    }
    if (boundDataSet != null) {
      boundDataSet.removeNavigationListener(this);
      boundDataSet.removeAccessListener(this);
      removeItemListener(this);
    }
    boundDataSet = newDataSet;
    if (boundDataSet != null) {
      if (!boundDataSet.isOpen()) {
        try {
          boundDataSet.open();
        }
        catch (DataSetException ex) {
          DataSetException.handleException(boundDataSet, this, ex);
          return;
        }
      }
      boundDataSet.addAccessListener(this);
      boundDataSet.addNavigationListener(this);
      addItemListener(this);
      try {
        bindDataSet();
        repaint();
      }
      catch (DataSetException x) {
      }
    }
    else {
      //super.setModel(new NullComboBoxModel());
      repaint();
    }
  }

  private void bindDataSet() throws DataSetException {
    Column column;
    if (boundDataSet != null && boundDataSet.isOpen() && (column = boundDataSet.hasColumn(columnName)) != null) {
      PickListDescriptor pickList = column.getPickList();
      if (pickList != null) {
        DataSet pds = pickList.getPickListDataSet();
        String[] dispCols = pickList.getPickListDisplayColumns();
        Object[] items = new String[pds.getRowCount()];
        pds.first();
        while (!pds.atLast()) {
          Variant value = new Variant();
          pds.getVariant(dispCols[0], value);
          items[pds.getRow()] = value;
          pds.next();
        }
        setItems(items);
      }
    }
  }

  // ItemListener Implementation

  public void itemStateChanged(ItemEvent e) {
    if (navigating) {
      return;
    }
    if (boundDataSet != null && boundDataSet.isOpen() && boundDataSet.hasColumn(columnName) != null) {
      int index = getSelectedIndex();
      Object data = getModel().getElementAt(index);
      try {
        if (data instanceof Variant) {
          Variant value = (Variant)data;
          boundDataSet.setVariant(columnName, value);
        }
        else {
          boundDataSet.setString(columnName, data != null ? data.toString() : "");
        }
      }
      catch (DataSetException x) {
      }
    }
  }

  // NavigationListener Implementation (DataSet)

  public void navigated(NavigationEvent e) {
    try {
      Variant value = new Variant();
      boundDataSet.getVariant(columnName, value);
      ListModel lm = getModel();
      for (int i = 0; i < lm.getSize(); i++) {
        if (value.equals(lm.getElementAt(i)) || value.toString().equals(lm.getElementAt(i))) {
          navigating = true;
          setSelectedIndex(i);
          navigating = false;
          return;
        }
      }
    }
    catch (DataSetException x) {
    }
  }

  // AccessListener Implementation (DataSet)

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      default:
        openDataSet(null);
        break;
    }
  }

  protected DataSet dataSet;
  protected DataSet boundDataSet;
  protected String  columnName;
  private boolean navigating = true;
  private boolean addNotifyCalled;
}

