/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.text.*;
import borland.jbcl.util.*;
import borland.jbcl.model.*;
import borland.jbcl.dataset.*;

public class JdbLabel extends JLabel
  implements NavigationListener, DataChangeListener, AccessListener,
             ColumnAware, java.io.Serializable
{
  public JdbLabel(String text, Icon icon, int horizontalAlignment) {
    super(text, icon, horizontalAlignment);
  }

  public JdbLabel(String text, int horizontalAlignment) {
    super(text, horizontalAlignment);
  }

  public JdbLabel(String text) {
    super(text);
  }

  public JdbLabel(Icon image, int horizontalAlignment) {
    super(image, horizontalAlignment);
  }

  public JdbLabel(Icon image) {
    super(image);
  }

  public JdbLabel() {
    super();
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to display data from in the field.
   */
  public void setDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeAccessListener(this);
      dataSet.removeNavigationListener(this);
      dataSet.removeDataChangeListener(this);
    }
    openDataSet(newDataSet);
    if (dataSet != null) {
      dataSet.addAccessListener(this);
      dataSet.addNavigationListener(this);
      dataSet.addDataChangeListener(this);
    }
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String newColumnName) {
    columnName = newColumnName;
    if (addNotifyCalled)
      openDataSet(dataSet);
  }
  public String getColumnName() {
    return columnName;
  }

  String current = "";
  protected void updateText() {
    Column column;
    if (dataSet != null && dataSet.isOpen() && (column = dataSet.hasColumn(columnName)) != null) {
      try {
        Variant data = new Variant();
        dataSet.getVariant(columnName, data);
        String text = null;
        ItemFormatter formatter = column.getFormatter();
        if (formatter != null)
          text = formatter.format(data);
        else
          text = data != null ? data.toString() : "";
        super.setText(text);
        current = text;
      }
      catch (Exception x) {
        DataSetException.handleException(dataSet, this, x);
        return;
      }
    }
  }

  // AccessListener Implementation

  public void accessChange(AccessEvent e) {
    switch(e.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          e.appendException(ex);
        }
        break;
      case AccessEvent.CLOSE:
        break;
      default:
        break;
    }
  }

  // NavigationListener Implementation

  public void navigated(NavigationEvent e) { updateText(); }

  // DataChangeListener Implementation

  public void dataChanged(DataChangeEvent e) { updateText(); }
  public void postRow(DataChangeEvent e) throws Exception {}

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null && columnName != null && !"".equals(columnName))
        openDataSet(dataSet);
    }
  }

  private void openDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    if (dataSet == null) {
      return;
    }
    else if (addNotifyCalled && !dataSet.isOpen()) {
      try {
        dataSet.open();
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
        return;
      }
    }
    if (dataSet.isOpen()) {
      bindDataSet();
      updateText();
    }
  }

  private void bindDataSet() {
    Column column;
    if (dataSet != null && (column = dataSet.hasColumn(columnName)) != null) {
      bindProperties(column);
    }
  }

  private void bindProperties(Column column) {
    Color bg = column.getBackground();
    Color fg = column.getForeground();
    Font  f  = column.getFont();
    if (bg != null) setBackground(bg);
    if (fg != null) setForeground(fg);
    if (f != null)  setFont(f);
  }

  private DataSet dataSet;
  private String columnName;
  private boolean addNotifyCalled = false;
}

