/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import borland.jbcl.util.*;
import borland.jbcl.dataset.*;
import java.util.Vector;

/**
 * The JdbList is a data-aware extension to the JList component.  JdbList
 * displays a list of pick-values for a single field in a DataSet.
 */
public class JdbList extends JList
  implements ListSelectionListener, NavigationListener,
             AccessListener, DataChangeListener, ColumnAware
{
  public JdbList(ListModel dataModel) {
    super(dataModel);
  }

  public JdbList(final Object[] listData) {
    super(listData);
  }

  public JdbList(final Vector listData) {
    super(listData);
  }

  public JdbList() {
    super();
  }

  /**
   * The items property fills the list with a list of items
   * to pick from.
   */
  public synchronized void setItems(String[] items) {
    if (items != null) {
      super.setListData(items);
    }
    else {
      super.setModel(new NullComboBoxModel());
    }
    repaint();
  }
  public synchronized void setItems(Object[] items) {
    if (items != null) {
      super.setListData(items);
    }
    else {
      super.setModel(new NullComboBoxModel());
    }
    repaint();
  }

  public synchronized String[] getItems() {
    if (getModel() == null || getModel().getSize() == 0)
      return null;
    String[] items = new String[getModel().getSize()];
    for (int i = 0; i < getModel().getSize(); i++) {
      Object data = getModel().getElementAt(i);
      items[i] = data != null ? data.toString() : "";
    }
    return items;
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to drive the list's selection.
   */
  public void setDataSet(DataSet dataSet) {
    openDataSet(dataSet);
    repaint();
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  /**
   * The columnName property specifies a column name in the dataSet
   * object to drive the list's selection.
   */
  public void setColumnName(String columnName) {
    this.columnName = columnName;
    openDataSet(dataSet);
    repaint();
  }
  public String getColumnName() {
    return columnName;
  }

  private void openDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeNavigationListener(this);
      dataSet.removeDataChangeListener(this);
      dataSet.removeAccessListener(this);
      getSelectionModel().removeListSelectionListener(this);
    }
    dataSet = newDataSet;
    if (dataSet != null) {
      if (addNotifyCalled && !dataSet.isOpen()) {
        try {
          dataSet.open();
        }
        catch (DataSetException ex) {
          DataSetException.handleException(dataSet, this, ex);
          return;
        }
      }
      if (dataSet.isOpen()) {
        dataSet.addAccessListener(this);
        dataSet.addNavigationListener(this);
        dataSet.addDataChangeListener(this);
        getSelectionModel().addListSelectionListener(this);
        try {
          bindDataSet();
          repaint();
        }
        catch (DataSetException x) {
        }
      }
    }
  }

  private void bindDataSet() throws DataSetException {
    Column column;
    if (dataSet != null && (column = dataSet.hasColumn(columnName)) != null) {
      PickListDescriptor pickList = column.getPickList();
      if (pickList != null) {
        DataSet pds = pickList.getPickListDataSet();
        String[] dispCols = pickList.getPickListDisplayColumns();
        Object[] items = new Object[pds.getRowCount()];
        pds.first();
        while (!pds.atLast()) {
          Variant value = new Variant();
          pds.getVariant(dispCols[0], value);
          items[pds.getRow()] = value;
          pds.next();
        }
        setItems(items);
      }
    }
    Graphics g = getGraphics();
    if (g != null) {
      Font f = getFont();
      FontMetrics fm = getGraphics().getFontMetrics(f);
      setFixedCellHeight(fm.getHeight());
    }
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
//    setSelectionBackground(SystemColor.textHighlight);
//    setSelectionForeground(SystemColor.textHighlightText);
    invalidate();
    Container p = getParent();
    if (p != null && p.isShowing())
      p.validate();
  }

  public void addNotify() {
    super.addNotify();
    Graphics g = getGraphics();
    if (g != null) {
      Font f = getFont();
      FontMetrics fm = getGraphics().getFontMetrics(f);
      setFixedCellHeight(fm.getHeight());
    }
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null)
        openDataSet(dataSet);
    }
  }

  // ListSelectionListener Implementation

  public void valueChanged(ListSelectionEvent e) {
    if (e.getValueIsAdjusting() || updating) {
      return;
    }
    if (dataSet != null && dataSet.hasColumn(columnName) != null) {
      int index = getSelectedIndex();
      Object data = getModel().getElementAt(index);
      try {
        if (data instanceof Variant)
          dataSet.setVariant(columnName, (Variant)data);
        else {
          dataSet.setString(columnName, data != null ? data.toString() : "");
        }
      }
      catch (DataSetException x) {
      }
    }
  }

  // NavigationListener Implementation (DataSet)

  public void navigated(NavigationEvent e) {
    updateValue();
  }

  // DataChangeListener Implementation (DataSet)

  public void dataChanged(DataChangeEvent e) {
    updateValue();
  }
  public void postRow(DataChangeEvent e) {}

  public void updateValue() {
    try {
      Variant value = new Variant();
      dataSet.getVariant(columnName, value);
      ListModel lm = getModel();
      for (int i = 0; i < lm.getSize(); i++) {
        if (value.equals(lm.getElementAt(i)) || value.toString().equals(lm.getElementAt(i))) {
          updating = true;
          setSelectedIndex(i);
          ensureIndexIsVisible(i);
          updating = false;
          return;
        }
        updating = true;
        clearSelection();
        updating = false;
      }
    }
    catch (DataSetException x) {
    }
  }

  // AccessListener Implementation (DataSet)

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
          repaint();
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      default:
        //super.setModel(new NullComboBoxModel());
        repaint();
        break;
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 5 || ps.height < 5) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }
    return ps;
  }

  protected DataSet dataSet;
  protected String  columnName;
  private boolean updating;
  private boolean addNotifyCalled;
}
