/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import borland.jbcl.util.*;
import borland.jbcl.model.*;
import borland.jbcl.dataset.*;
import java.util.Vector;

public class JdbNavComboBox extends JComboBox
  implements NavigationListener, AccessListener,
             ColumnAware, java.io.Serializable
{
  public JdbNavComboBox() {
    super();
  }

  public JdbNavComboBox(ComboBoxModel aModel) {
    super(aModel);
  }

  public JdbNavComboBox(DataSet dataSet, String columnName) {
    super();
    this.columnName = columnName;
    openDataSet(dataSet);
    setLightWeightPopupEnabled(false);
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet(dataSet);
      }
    }
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to fill the list contents.
   */
  public void setDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    openDataSet(dataSet);
    repaint();
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  /**
   * The columnName property specifies a column name in the dataSet
   * object to fill the list contents.
   */
  public void setColumnName(String columnName) {
    this.columnName = columnName;
    openDataSet(dataSet);
    repaint();
  }
  public String getColumnName() {
    return columnName;
  }

  private void openDataSet(DataSet newDataSet) {
    if (!addNotifyCalled) {
      return;
    }
    if (boundDataSet != null) {
      boundDataSet.removeNavigationListener(this);
      boundDataSet.removeAccessListener(this);
    }
    boundDataSet = newDataSet;
    if (boundDataSet != null) {
      if (!boundDataSet.isOpen()) {
        try {
          boundDataSet.open();
        }
        catch (DataSetException ex) {
          DataSetException.handleException(boundDataSet, this, ex);
          return;
        }
      }
      boundDataSet.addAccessListener(this);
      boundDataSet.addNavigationListener(this);
      bindDataSet();
    }
    else {
      super.setModel(new NullComboBoxModel());
      repaint();
    }
  }

  private void bindDataSet() {
    if (boundDataSet != null && (boundDataSet.hasColumn(columnName)) != null) {
      dbNavComboBoxModel dm = new dbNavComboBoxModel(boundDataSet, columnName, this);
      super.setModel(dm);
      repaint();
    }
    else {
      super.setModel(new NullComboBoxModel());
      repaint();
    }
  }

  // NavigationListener Implementation (DataSet)

  public void navigated(NavigationEvent e) {
    repaint();
  }

  // AccessListener Implementation (DataSet)

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      default:
        super.setModel(new NullComboBoxModel());
        break;
    }
  }

  protected DataSet dataSet;
  protected DataSet boundDataSet;
  protected String columnName;
  private boolean addNotifyCalled;
}

