/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import borland.jbcl.dataset.*;
import java.util.Vector;

public class JdbNavList extends JList
  implements NavigationListener, AccessListener, ListSelectionListener,
             ColumnAware, java.io.Serializable
{
  public JdbNavList(ListModel dataModel) {
    super(dataModel);
  }

  public JdbNavList(final Object[] listData) {
    super(listData);
  }

  public JdbNavList(final Vector listData) {
    super(listData);
  }

  public JdbNavList() {
    super();
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to fill the list contents.
   */
  public void setDataSet(DataSet dataSet) {
    openDataSet(dataSet);
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  /**
   * The columnName property specifies a column name in the dataSet
   * object to fill the list contents.
   */
  public void setColumnName(String columnName) {
    this.columnName = columnName;
    openDataSet(dataSet);
    repaint();
  }
  public String getColumnName() {
    return columnName;
  }

  private void openDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeNavigationListener(this);
      dataSet.removeAccessListener(this);
      getSelectionModel().removeListSelectionListener(this);
    }
    dataSet = newDataSet;
    if (dataSet != null) {
      if (addNotifyCalled && !dataSet.isOpen()) {
        try {
          dataSet.open();
        }
        catch (DataSetException ex) {
          DataSetException.handleException(dataSet, this, ex);
          return;
        }
      }
      if (dataSet.isOpen()) {
        dataSet.addAccessListener(this);
        dataSet.addNavigationListener(this);
        getSelectionModel().addListSelectionListener(this);
        bindDataSet();
        repaint();
      }
    }
  }

  private void bindDataSet() {
    if (dataSet != null && (dataSet.hasColumn(columnName)) != null) {
      dbListModel dm = new dbListModel(dataSet, columnName, this);
      setModel(dm);
      Graphics g = getGraphics();
      if (g != null) {
        Font f = getFont();
        FontMetrics fm = g.getFontMetrics(f);
        setFixedCellHeight(fm.getHeight());
      }
      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
//      setSelectionBackground(SystemColor.textHighlight);
//      setSelectionForeground(SystemColor.textHighlightText);
      invalidate();
      Container p = getParent();
      if (p != null && p.isShowing())
        p.validate();
    }
  }

  public void addNotify() {
    super.addNotify();
    Graphics g = getGraphics();
    if (g != null) {
      Font f = getFont();
      FontMetrics fm = g.getFontMetrics(f);
      setFixedCellHeight(fm.getHeight());
    }
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null)
        openDataSet(dataSet);
    }
  }

  // ListSelectionListener Implementation

  boolean navigating = false;
  public void valueChanged(ListSelectionEvent e) {
    if (navigating) {
      navigating = false;
      return;
    }
    int index = getSelectedIndex();
    if (dataSet.getRow() != index) {
      try {
        dataSet.goToRow(index);
      }
      catch (DataSetException x) {
      }
    }
  }

  // NavigationListener Implementation (DataSet)

  public void navigated(NavigationEvent e) {
    int row = dataSet.getRow();
    if (getSelectedIndex() != row) {
      navigating = true;
      setSelectedIndex(row);
    }
    ensureIndexIsVisible(row);
  }

  // AccessListener Implementation (DataSet)

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      default:
        super.setModel(new NullComboBoxModel());
        break;
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 5 || ps.height < 5) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }
    return ps;
  }

  protected DataSet dataSet;
  protected String columnName;
  protected boolean addNotifyCalled;
}
