/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import java.net.*;
import java.awt.event.*;
import borland.jbcl.util.*;
import borland.jbcl.dataset.*;
import com.sun.java.swing.*;

public class JdbNavToolBar
     extends JToolBar
  implements AccessListener, NavigationListener, StatusListener,
             DataSetAware, java.io.Serializable
{
  public final Action FIRST   = new dbFirstAction(this);
  public final Action PRIOR   = new dbPriorAction(this);
  public final Action NEXT    = new dbNextAction(this);
  public final Action LAST    = new dbLastAction(this);
  public final Action INSERT  = new dbInsertAction(this);
  public final Action DELETE  = new dbDeleteAction(this);
  public final Action POST    = new dbPostAction(this);
  public final Action CANCEL  = new dbCancelAction(this);
  public final Action DITTO   = new dbDittoAction(this);
  public final Action SAVE    = new dbSaveAction(this);
  public final Action REFRESH = new dbRefreshAction(this);

  public JdbNavToolBar() {
    super();
    setLayout(new GridLayout());
    add(FIRST);
    add(PRIOR);
    add(NEXT);
    add(LAST);
    add(INSERT);
    add(DELETE);
    add(POST);
    add(CANCEL);
    add(DITTO);
    add(SAVE);
    add(REFRESH);
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null)
        openDataSet(dataSet);
    }
  }

  public DataSet getDataSet() { return dataSet; }
  public void setDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeAccessListener(this);
      dataSet.removeNavigationListener(this);
      dataSet.removeStatusListener(this);
    }
    openDataSet(newDataSet);
    if (dataSet != null) {
      dataSet.addAccessListener(this);
      dataSet.addNavigationListener(this);
      dataSet.addStatusListener(this);
    }
  }

  private void openDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    if (dataSet == null) {
      updateSelection();
      return;
    }
    else if (addNotifyCalled && !dataSet.isOpen()) {
      try {
        dataSet.open();
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
        return;
      }
    }
    if (dataSet.isOpen()) {
      SAVE.setEnabled(dataSet.saveChangesSupported());
      REFRESH.setEnabled(dataSet.refreshSupported());
      updateSelection();
    }
  }

  // AccessListener method

  boolean closing = false;
  public void accessChange(AccessEvent event) {
    switch (event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      case AccessEvent.CLOSE:
        closing = true;
        updateSelection();
        closing = false;
        break;
      default:
        break;
    }
  }

  // StatusListener method

  public void statusMessage(StatusEvent event) {
    try {
      switch (event.getCode()) {
        case (StatusEvent.EDIT_STARTED):
        case (StatusEvent.EDIT_CANCELED):
        case (StatusEvent.DATA_CHANGE):
          INSERT.setEnabled(dataSet != null && dataSet.isOpen() && !dataSet.isEditingNewRow() && dataSet.isEnableInsert());
          DELETE.setEnabled(dataSet != null && dataSet.isOpen() && !dataSet.isEmpty() && dataSet.isEnableDelete());
          POST.setEnabled(dataSet != null && dataSet.isOpen() && dataSet.isEditing());
          CANCEL.setEnabled(dataSet != null && dataSet.isOpen() && dataSet.isEditing());
          DITTO.setEnabled(dataSet != null && dataSet.isOpen() && dataSet.isEditingNewRow());
          break;
      }
    }
    catch(DataSetException ex) {
      DataSetException.handleException(dataSet, this, ex);
    }
  }

  // NavigationListener method

  public void navigated(NavigationEvent event) { updateSelection(); }


  private void updateSelection() {
    if (!closing && dataSet != null && dataSet.isOpen()) {
      try {
        FIRST.setEnabled(!(dataSet.atFirst() || dataSet.isEmpty()));
        PRIOR.setEnabled(!(dataSet.atFirst() || dataSet.isEmpty()));
        NEXT.setEnabled(!(dataSet.atLast() && dataSet.isEditingNewRow()));
        LAST.setEnabled(!(dataSet.atLast() || dataSet.isEmpty()));
        INSERT.setEnabled(dataSet.isEnableInsert());
        DELETE.setEnabled(!dataSet.isEmpty() && dataSet.isEnableDelete());
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
      }
    }
    else {
      FIRST.setEnabled(false);
      PRIOR.setEnabled(false);
      NEXT.setEnabled(false);
      LAST.setEnabled(false);
      INSERT.setEnabled(false);
      DELETE.setEnabled(false);
      POST.setEnabled(false);
      CANCEL.setEnabled(false);
      DITTO.setEnabled(false);
      SAVE.setEnabled(false);
      REFRESH.setEnabled(false);
    }
  }

  protected DataSet dataSet;
  protected boolean addNotifyCalled;
}

class dbFirstAction extends dbAction {
  public dbFirstAction(DataSetAware dsa) {
    super(dsa, "First", "image/first.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.first(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbPriorAction extends dbAction {
  public dbPriorAction(DataSetAware dsa) {
    super(dsa, "Prior", "image/prior.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.prior(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbNextAction extends dbAction {
  public dbNextAction(DataSetAware dsa) {
    super(dsa, "Next", "image/next.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try {
        if (ds.atLast())
          ds.insertRow(false);
        else
          ds.next();
      }
      catch (DataSetException x) {
        DataSetException.handleException(ds, null, x);
      }
    }
  }
}

class dbLastAction extends dbAction {
  public dbLastAction(DataSetAware dsa) {
    super(dsa, "Last", "image/last.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.last(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbInsertAction extends dbAction {
  public dbInsertAction(DataSetAware dsa) {
    super(dsa, "Insert", "image/insert.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.insertRow(true); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
};

class dbDeleteAction extends dbAction {
  public dbDeleteAction(DataSetAware dsa) {
    super(dsa, "Delete", "image/delete.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.deleteRow(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbPostAction extends dbAction {
  public dbPostAction(DataSetAware dsa) {
    super(dsa, "Post", "image/post.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.post(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbCancelAction extends dbAction {
  public dbCancelAction(DataSetAware dsa) {
    super(dsa, "Cancel", "image/cancel.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try {
        if (ds.isEditing())
          ds.cancel();
        else
          ds.cancelLoading();
      }
      catch (DataSetException x) {
        DataSetException.handleException(ds, null, x);
      }
    }
  }
}

class dbDittoAction extends dbAction {
  public dbDittoAction(DataSetAware dsa) {
    super(dsa, "Ditto", "image/ditto.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.dittoRow(false); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbSaveAction extends dbAction {
  public dbSaveAction(DataSetAware dsa) {
    super(dsa, "Save", "image/save.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.saveChanges(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

class dbRefreshAction extends dbAction {
  public dbRefreshAction(DataSetAware dsa) {
    super(dsa, "Refresh", "image/refresh.gif");
  }
  public void actionPerformed(ActionEvent e) {
    DataSet ds = dsa.getDataSet();
    if (ds != null && ds.isOpen()) {
      try { ds.refresh(); }
      catch (DataSetException x) { DataSetException.handleException(ds, null, x); }
    }
  }
}

abstract class dbAction extends AbstractAction {
  public dbAction(DataSetAware dsa, String name, String iconResource) {
    super(name);
    this.dsa = dsa;
    try {
      URL url = getClass().getResource(iconResource);
      super.putValue(SMALL_ICON, new ImageIcon(url));
    }
    catch (Exception x) {
    }
  }
  protected DataSetAware dsa;
}
