/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import com.sun.java.swing.*;
import borland.jbcl.util.*;
import borland.jbcl.dataset.*;
import java.awt.*;
import java.awt.event.*;

public class JdbRadioButton extends JRadioButton
  implements AccessListener, ColumnAware, java.io.Serializable
{
  public JdbRadioButton () {
    super();
  }

  public JdbRadioButton(Icon icon) {
    super(icon);
  }

  public JdbRadioButton(Icon icon, boolean selected) {
    super(icon, selected);
  }

  public JdbRadioButton (String text) {
    super(text);
  }

  public JdbRadioButton (String text, boolean selected) {
    super(text, selected);
  }

  public JdbRadioButton(String text, Icon icon) {
    super(text, icon);
  }

  public JdbRadioButton (String text, Icon icon, boolean selected) {
    super(text, icon, selected);
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to display data from in the field.
   */
  public void setDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeAccessListener(this);
    }
    openDataSet(newDataSet);
    if (dataSet != null) {
      dataSet.addAccessListener(this);
    }
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String newColumnName) {
    columnName = newColumnName;
    if (addNotifyCalled)
      openDataSet(dataSet);
  }
  public String getColumnName() {
    return columnName;
  }

  // AccessListener Implementation

  public void accessChange(AccessEvent e) {
    switch(e.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          e.appendException(ex);
        }
        break;
      case AccessEvent.CLOSE:
        ButtonModel bm = getModel();
        if (bm instanceof dbRadioButtonModel) {
          ((dbRadioButtonModel)bm).close();
        }
        setModel(new ToggleButtonModel());
        break;
      default:
        break;
    }
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null && columnName != null && !"".equals(columnName))
        openDataSet(dataSet);
    }
  }

  private void openDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    if (dataSet == null) {
      return;
    }
    else if (addNotifyCalled && !dataSet.isOpen()) {
      try {
        dataSet.open();
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
        return;
      }
    }
    if (dataSet.isOpen()) {
      bindDataSet();
    }
  }

  private void bindDataSet() {
    Column column;
    if (dataSet != null && (column = dataSet.hasColumn(columnName)) != null) {
      setModel(new dbRadioButtonModel(dataSet, columnName));
      bindProperties(column);
    }
  }

  private void bindProperties(Column column) {
    String cap = column.getCaption();
    Color bg = column.getBackground();
    Color fg = column.getForeground();
    Font  f  = column.getFont();
    if (cap != null) setText(cap);
    if (bg != null) setBackground(bg);
    if (fg != null) setForeground(fg);
    if (f != null)  setFont(f);
  }

  private DataSet dataSet;
  private String columnName;
  private boolean addNotifyCalled = false;
}

class dbRadioButtonModel extends DefaultButtonModel
implements NavigationListener, DataChangeListener
{
  public dbRadioButtonModel(DataSet dataSet, String columnName) {
    super();
    this.dataSet = dataSet;
    this.columnName = columnName;
    dataSet.addNavigationListener(this);
    dataSet.addDataChangeListener(this);
  }

  public void close() {
    dataSet.removeNavigationListener(this);
    dataSet.removeDataChangeListener(this);
    dataSet = null;
    columnName = null;
  }

  public void setPressed(boolean b) {
    if ((isPressed() == b) || !isEnabled())
      return;
    if (b == false && isArmed())
      setSelected(!this.isSelected());
    if (b)
      stateMask |= PRESSED;
    else
      stateMask &= ~PRESSED;
    fireStateChanged();
    if(!isPressed() && isArmed())
      fireActionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, getActionCommand()));
  }

  boolean settingValue = false;
  public void setSelected(boolean b) {
    if (this.isSelected() == b)
      return;
    if (dataSet != null && dataSet.isOpen() && dataSet.hasColumn(columnName) != null) {
      Variant value = new Variant();
      try {
        dataSet.getVariant(columnName, value);
        int type = value.getType();
        if (type == Variant.UNASSIGNED_NULL || type == Variant.ASSIGNED_NULL)
          type = dataSet.hasColumn(columnName).getDataType();
        switch (type) {
          case Variant.BOOLEAN: value.setBoolean(b);                break;
          case Variant.STRING:  value.setString(String.valueOf(b)); break;
          case Variant.BYTE:    value.setByte((byte)(b ? 1 : 0));   break;
          case Variant.SHORT:   value.setShort((short)(b ? 1 : 0)); break;
          case Variant.INT:     value.setInt(b ? 1 : 0);            break;
          case Variant.LONG:    value.setLong(b ? 1 : 0);           break;
          case Variant.FLOAT:   value.setFloat(b ? 1 : 0);          break;
          case Variant.DOUBLE:  value.setDouble(b ? 1 : 0);         break;
          default: value.setUnassignedNull();
        }
        if (!value.isUnassignedNull()) {
          if (!dataSet.isEditing())
            dataSet.startEdit(dataSet.hasColumn(columnName));
          settingValue = true;
          dataSet.setVariant(columnName, value);
          settingValue = false;
        }
      }
      catch (DataSetException x) {
        x.printStackTrace();
        settingValue = false;
        return;
      }
    }
    super.setSelected(b);
  }

  public void checkState() {
    if (dataSet != null && dataSet.isOpen() && dataSet.hasColumn(columnName) != null) {
      Variant value = new Variant();
      try {
        dataSet.getVariant(columnName, value);
        boolean b = value.getAsBoolean();
        if (super.isSelected() != b)
          super.setSelected(b);
      }
      catch (DataSetException x) {
        x.printStackTrace();
      }
    }
  }

  public void navigated(NavigationEvent event) {
    // this will update the model
    if (!settingValue)
      checkState();
  }

  // DataChangeListener Implementation

  public void dataChanged(DataChangeEvent e) {
    // this will update the model
    if (!settingValue)
      checkState();
  }
  public void postRow(DataChangeEvent e) throws Exception {}

  protected DataSet dataSet;
  protected String columnName;
}

