/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import borland.jbcl.dataset.*;
import com.sun.java.swing.*;

/**
 * Data aware JdbStatusLabel component.  If DataSet property is set, the JdbStatusLabel displays
 * error and status messages related to operations on the DataSet.
 */
public class JdbStatusLabel
     extends JLabel
  implements NavigationListener, StatusListener, AccessListener,
             DataSetAware, java.io.Serializable
{
  public JdbStatusLabel() {
    super();
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null)
        openDataSet(dataSet);
    }
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to display data from in the JdbStatusLabel.
   */
  public DataSet getDataSet() { return dataSet; }
  public void setDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeAccessListener(this);
      dataSet.removeNavigationListener(this);
      dataSet.removeStatusListener(this);
    }
    openDataSet(newDataSet);
    if (dataSet != null) {
      dataSet.addAccessListener(this);
      dataSet.addNavigationListener(this);
      dataSet.addStatusListener(this);
    }
  }

  private void openDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    if (dataSet == null) {
      return;
    }
    else if (addNotifyCalled && !dataSet.isOpen()) {
      try {
        dataSet.open();
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
        return;
      }
    }
    if (dataSet.isOpen()) {
      updateValue();
    }
  }

  /** Implementation of StatusListener.
  */
  public void statusMessage(StatusEvent event) {
    setText(event.getMessage());
  }

  /** Implementation of AccessListener.
  */

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        updateValue();
        break;
      case AccessEvent.CLOSE:
        setText("");
        break;
    }
  }

  /** Implementation of NavigationListener.
  */
  public void navigated(NavigationEvent event) { updateValue(); }

  protected void updateValue() {
    if (dataSet != null && dataSet.isOpen()) {
      try {
        if (dataSet.getRowCount() > 0)
          setText("Record " + (dataSet.getRow() + 1) + " of " + dataSet.getRowCount());
        else
          setText("Empty DataSet");
      }
      catch (Exception x) {
        x.printStackTrace();
      }
    }
  }

  protected DataSet dataSet;
  private boolean addNotifyCalled = false;
}
