/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.util.Vector;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;
import borland.jbcl.dataset.*;

public class JdbTable extends JTable
  implements NavigationListener, AccessListener, ListSelectionListener,
             DataSetAware, java.io.Serializable
{
  public JdbTable() {
    super();
  }

  public JdbTable(TableModel dm) {
    super(dm);
  }

  public JdbTable(TableModel dm, TableColumnModel cm) {
    super(dm, cm);
  }

  public JdbTable(TableModel dm, TableColumnModel cm, ListSelectionModel sm) {
    super(dm, cm, sm);
  }

  public JdbTable(int numColumns, int numRows) {
    super(numColumns, numRows);
  }

  public JdbTable(Vector data, Vector columnNames) {
    super(data, columnNames);
  }

  public JdbTable(Object[][] data, Object[] columnNames) {
    super(data, columnNames);
  }

  public void updateUI() {
    super.updateUI();
    Container p = getParent();
    if (p instanceof JScrollPane) {
      JScrollPane sp = (JScrollPane)p;
      sp.setColumnHeaderView(getTableHeader());
      sp.getViewport().setBackingStoreEnabled(false);
      sp.setBorder(UIManager.getBorder("Table.scrollPaneBorder"));
    }
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to fill the list contents.
   */
  public void setDataSet(DataSet dataSet) {
    openDataSet(dataSet);
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null)
        openDataSet(dataSet);
    }
    updateUI();
  }

  private void openDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeNavigationListener(this);
      dataSet.removeAccessListener(this);
      getSelectionModel().removeListSelectionListener(this);
    }
    dataSet = newDataSet;
    if (dataSet != null) {
      if (addNotifyCalled && !dataSet.isOpen()) {
        try {
          dataSet.open();
        }
        catch (DataSetException ex) {
          DataSetException.handleException(dataSet, this, ex);
          return;
        }
      }
      if (dataSet.isOpen()) {
        dataSet.addAccessListener(this);
        dataSet.addNavigationListener(this);
        getSelectionModel().addListSelectionListener(this);
        bindDataSet();
        repaint();
      }
    }
  }

  private void bindDataSet() {
    if (dataSet != null) {
      dbTableModel dm = new dbTableModel(dataSet, this);
      dbTableColumnModel dcm = new dbTableColumnModel(dataSet);
      setModel(dm);
      setColumnModel(dcm);
      Font f = getFont();
      Graphics g = getGraphics();
      if (g != null) {
        FontMetrics fm = g.getFontMetrics(f);
        setRowHeight(fm.getHeight());
      }
      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      setRowSelectionAllowed(false);
      setCellSelectionEnabled(true);
//      setSelectionBackground(SystemColor.textHighlight);
//      setSelectionForeground(SystemColor.textHighlightText);
      setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      setColumnSelectionInterval(0, 0);
      setRowSelectionInterval(dataSet.getRow(), dataSet.getRow());
      invalidate();
      Container p = getParent();
      if (p != null && p.isShowing())
        p.validate();
    }
  }

  // ListSelectionListener Implementation

  public void valueChanged(ListSelectionEvent e) {
    if (navigating) {
      navigating = false;
      return;
    }
    int row = getSelectedRow();
    if (dataSet.getRow() != row) {
      try {
        dataSet.goToRow(row);
      }
      catch (DataSetException x) {
      }
    }
  }

  // NavigationListener Implementation (DataSet)

  boolean navigating = false;
  public void navigated(NavigationEvent e) {
    int row = dataSet.getRow();
    navigating = true;
    int c = getSelectedColumn();
    int r = getSelectedRow();
    setRowSelectionInterval(row, row);
    Rectangle rect = getCellRect(row, c, true);
    scrollRectToVisible(rect);
    repaint();
  }

  // AccessListener Implementation (DataSet)

  public void accessChange(AccessEvent event) {
    switch(event.getID()) {
      case AccessEvent.OPEN:
        try {
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          event.appendException(ex);
        }
        break;
      default:
        super.setModel(new NullTableModel());
        break;
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 5 || ps.height < 5) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }
    return ps;
  }

  protected DataSet dataSet;
  protected boolean addNotifyCalled;
}
