/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.dbswing;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.text.*;
import borland.jbcl.util.*;
import borland.jbcl.model.*;
import borland.jbcl.dataset.*;

public class JdbTextPane extends JTextPane
  implements NavigationListener, DataChangeListener, AccessListener,
             ColumnAware, java.io.Serializable
{
  public JdbTextPane() {
    super();
  }

  public JdbTextPane(StyledDocument doc) {
    super(doc);
  }

  /**
   * The dataSet property specifies a borland.jbcl.dataset.DataSet
   * object to display data from in the field.
   */
  public void setDataSet(DataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeAccessListener(this);
      dataSet.removeNavigationListener(this);
      dataSet.removeDataChangeListener(this);
    }
    openDataSet(newDataSet);
    if (dataSet != null) {
      dataSet.addAccessListener(this);
      dataSet.addNavigationListener(this);
      dataSet.addDataChangeListener(this);
    }
  }
  public DataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String newColumnName) {
    columnName = newColumnName;
    if (addNotifyCalled)
      openDataSet(dataSet);
  }
  public String getColumnName() {
    return columnName;
  }

  static String current = "";
  protected void updateText() {
    Column column;
    if (dataSet != null && dataSet.isOpen() && (column = dataSet.hasColumn(columnName)) != null) {
      try {
        Variant data = new Variant();
        dataSet.getVariant(columnName, data);
        String text = null;
        ItemFormatter formatter = column.getFormatter();
        if (formatter != null)
          text = formatter.format(data);
        else
          text = data != null ? data.toString() : "";
        super.setText(text);
        current = text;
      }
      catch (Exception x) {
        DataSetException.handleException(dataSet, this, x);
        return;
      }
    }
  }

  protected boolean postText() {
    Column column;
    if (dataSet != null && dataSet.isOpen() && (column = dataSet.hasColumn(columnName)) != null) {
      String text = super.getText();
      if (text.equals(current))
        return true;
      try {
        ItemFormatter formatter = column.getFormatter();
        Variant data = new Variant();
        if (formatter != null)
          data = (Variant)formatter.parse(text);
        else
          data.setString(text);
        dataSet.setVariant(columnName, data);
      }
      catch (Exception x) {
        DataSetException.handleException(dataSet, this, x);
        return false;
      }
    }
    return true;
  }

  // AccessListener Implementation

  public void accessChange(AccessEvent e) {
    switch(e.getID()) {
      case AccessEvent.OPEN:
        try {
          setEditable(true);
          openDataSet(dataSet);
        }
        catch (Exception ex) {
          e.appendException(ex);
        }
        break;
      case AccessEvent.CLOSE:
        setEditable(false);
        break;
      default:
        break;
    }
  }

  // NavigationListener Implementation

  public void navigated(NavigationEvent e) { updateText(); }

  // DataChangeListener Implementation

  public void dataChanged(DataChangeEvent e) { updateText(); }
  public void postRow(DataChangeEvent e) throws Exception { postText(); }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null && columnName != null && !"".equals(columnName))
        openDataSet(dataSet);
    }
  }

  private void openDataSet(DataSet newDataSet) {
    dataSet = newDataSet;
    if (dataSet == null) {
      return;
    }
    else if (addNotifyCalled && !dataSet.isOpen()) {
      try {
        dataSet.open();
      }
      catch (DataSetException ex) {
        DataSetException.handleException(dataSet, this, ex);
        return;
      }
    }
    if (dataSet.isOpen()) {
      bindDataSet();
      updateText();
    }
  }

  private void bindDataSet() {
    Column column;
    if (dataSet != null && (column = dataSet.hasColumn(columnName)) != null) {
      bindProperties(column);
    }
  }

  private void bindProperties(Column column) {
    Color bg = column.getBackground();
    Color fg = column.getForeground();
    Font  f  = column.getFont();
    if (bg != null) setBackground(bg);
    if (fg != null) setForeground(fg);
    if (f != null)  setFont(f);
  }

  // Override key events for DataSet specific actions.
  protected void processKeyEvent(KeyEvent e) {
    super.processKeyEvent(e);
    if (dataSet == null || !dataSet.isOpen() || dataSet.hasColumn(columnName) == null)
      return;
    switch (e.getID()) {
      case e.KEY_TYPED :
        String text = super.getText();
        if (!text.equals(current) && !dataSet.isEditing()) {
          Column c = dataSet.hasColumn(columnName);
          try {
            dataSet.startEdit(c);
          }
          catch (DataSetException x) {
          }
        }
        break;
      case e.KEY_PRESSED :
        if (e.isConsumed())
          return;
        try {
          switch (e.getKeyCode()) {
            case KeyEvent.VK_ENTER :
              if (e.isControlDown())
                postText();
              break;
            case KeyEvent.VK_ESCAPE :
              updateText();
              break;
            case KeyEvent.VK_INSERT :
              if (e.isControlDown() && postText()) {
                dataSet.insertRow(true);
              }
              break;
            case KeyEvent.VK_DELETE :
              if (e.isControlDown())
                dataSet.deleteRow();
              break;
            case KeyEvent.VK_PAGE_DOWN :
              if (e.isControlDown() && postText()) {
                dataSet.next();
              }
              break;
            case KeyEvent.VK_PAGE_UP :
              if (e.isControlDown() && postText()) {
                dataSet.prior();
              }
              break;
          }
        }
        catch (DataSetException x) {
          x.printStackTrace();
        }
    }
  }

  protected void processFocusEvent(FocusEvent e) {
    super.processFocusEvent(e);
    if (e.getID() == FocusEvent.FOCUS_LOST) {
      String text = super.getText();
      if (!text.equals(current))
        postText();
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 10 || ps.height < 10) {
      ps = ps != null ? ps : new Dimension(100, 100);
      ps.width = ps.width > 100 ? ps.width : 100;
      ps.height = ps.height > 100 ? ps.height : 100;
    }
    return ps;
  }

  private DataSet dataSet;
  private String columnName;
  private boolean addNotifyCalled = false;
}
