/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.control;

import java.awt.*;
import java.awt.event.*;
import com.objectspace.jgl.Array;
import borland.jbcl.util.*;

public class ButtonDialog extends Dialog implements ActionListener, KeyListener, java.io.Serializable
{
  public static final String NONE_COMMAND     = "none";     //NORES: internal name
  public static final String OK_COMMAND       = "ok";       //NORES: internal name
  public static final String YES_COMMAND      = "yes";      //NORES: internal name
  public static final String NO_COMMAND       = "no";       //NORES: internal name
  public static final String CANCEL_COMMAND   = "cancel";   //NORES: internal name
  public static final String DONE_COMMAND     = "done";     //NORES: internal name
  public static final String HELP_COMMAND     = "help";     //NORES: internal name
  public static final String APPLY_COMMAND    = "apply";    //NORES: internal name
  public static final String NEXT_COMMAND     = "next";     //NORES: internal name
  public static final String PREVIOUS_COMMAND = "previous"; //NORES: internal name
  public static final String DETAILS_COMMAND  = "details";  //NORES: internal name

  // buttons to be passed in buttonSet & available in result
  public static final int NONE      = 0x000;  // until something else is set, or used by external code to veto a close
  public static final int OK        = 0x001;  // these five will close the dialog
  public static final int YES       = 0x002;
  public static final int NO        = 0x004;
  public static final int CANCEL    = 0x008;
  public static final int DONE      = 0x010;
  public static final int HELP      = 0x020;
  public static final int APPLY     = 0x040;
  public static final int NEXT      = 0x080;
  public static final int PREVIOUS  = 0x100;
  public static final int DETAILS   = 0x200;

  // pre-defined button sets
  public static final int OK_CANCEL       = OK | CANCEL;
  public static final int YES_NO          = YES | NO;
  public static final int YES_NO_CANCEL   = YES | NO | CANCEL;
  public static final int OK_CANCEL_APPLY = OK | CANCEL | APPLY;
  public static final int APPLY_DONE      = APPLY | DONE;
  public static final int ALL             = OK | YES | NO | CANCEL | DONE | HELP | APPLY | NEXT | PREVIOUS | DETAILS;

  protected Panel             buttonPanel;
  protected transient ActionMulticaster actionMulticaster;

  protected int     buttonSet;
  protected boolean centered     = true; // centered on screen
  protected boolean enterOK      = true; // allow the enter key to generate an OK command
  protected boolean escapeCancel = true; // allow the escape key to generate a CANCEL command
  protected ButtonDescriptor[]  buttonDescriptors;
  protected String[] labels;
  protected Panel    buttonPanelHolder; //so we can have same size buttons
  protected int      buttonAlignment = FlowLayout.CENTER;
  protected ButtonDescriptor result;
  protected Component centerPanel;

  protected static final ButtonDescriptor[] buttonSetToButtonDescriptors(int buttonSet) {
    Array array = new Array();

    if ((buttonSet&PREVIOUS) != 0)
      array.add(new ButtonDescriptor(PREVIOUS_COMMAND, Res.getString(Res.Previous), PREVIOUS, false));
    if ((buttonSet&NEXT) != 0)
      array.add(new ButtonDescriptor(NEXT_COMMAND, Res.getString(Res.NextButton), NEXT, false));
    if ((buttonSet&DETAILS) != 0)
      array.add(new ButtonDescriptor(DETAILS_COMMAND, Res.getString(Res.Details), DETAILS, false));

    if ((buttonSet&OK) != 0)
      array.add(new ButtonDescriptor(OK_COMMAND, Res.getString(Res.OK), OK, true));
    if ((buttonSet&YES) != 0)
      array.add(new ButtonDescriptor(YES_COMMAND, Res.getString(Res.Yes1), YES, true));
    if ((buttonSet&NO) != 0)
      array.add(new ButtonDescriptor(NO_COMMAND, Res.getString(Res.No1), NO, true));
    if ((buttonSet&CANCEL) != 0)
      array.add(new ButtonDescriptor(CANCEL_COMMAND, Res.getString(Res.Cancel1), CANCEL, true));
    if ((buttonSet&DONE) != 0)
      array.add(new ButtonDescriptor(DONE_COMMAND, Res.getString(Res.Done), DONE, true));
    if ((buttonSet&HELP) != 0)
      array.add(new ButtonDescriptor(HELP_COMMAND, Res.getString(Res.Help), HELP, false));
    if ((buttonSet&APPLY) != 0)
      array.add(new ButtonDescriptor(APPLY_COMMAND, Res.getString(Res.Apply), APPLY, false));

    ButtonDescriptor[] list = new ButtonDescriptor[array.size()];
    array.copyTo(list);
    return list;
  }

  protected ButtonDialog(Frame              frame,
                         String             title,
                         boolean            modal,
                         Component          centerPanel,
                         Panel              buttonPanel,
                         ButtonDescriptor[] buttonDescriptors)
  {
    super(frame, title, modal);

    this.buttonPanel = buttonPanel;

    if (buttonPanel == null) {
      buttonPanelHolder = new Panel();
      buttonPanelHolder.setLayout(new FlowLayout(FlowLayout.CENTER));
      this.buttonPanel = new Panel();
      this.buttonPanel.setLayout(new GridLayout(1,0,6,0)); 
      buttonPanelHolder.add(this.buttonPanel);
    }

    if (centerPanel != null) {
      setLayout(new BorderLayout());
      setBackground(SystemColor.control);
      add(centerPanel, BorderLayout.CENTER);
      if (buttonPanelHolder != null)
        add(buttonPanelHolder, BorderLayout.SOUTH);
      else
        add(this.buttonPanel, BorderLayout.SOUTH);
    }

    this.buttonSet = -1;
    setButtonSet(buttonDescriptors);

    // add ourselves as key listener to EVERY child!
    listenForKeys(this);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    pack();
  }

  protected ButtonDialog(Frame frame, String title, Component centerPanel, int buttonSet) {
    this(frame, title, true, centerPanel, null, buttonSetToButtonDescriptors(buttonSet));
  }

  public ButtonDialog(Frame frame, String title, Component centerPanel) {
    this(frame, title, centerPanel, OK);
  }

  public Component getCenterPanel() {
    return centerPanel;
  }

  public void setCenterPanel(Component panel) {
    if (centerPanel != null) {
      dropKeyListeners(this);
    }
    centerPanel = panel;
    if (centerPanel != null) {
      setLayout(new BorderLayout());
      setBackground(SystemColor.control);
      add(centerPanel, BorderLayout.CENTER);
      if (buttonPanelHolder != null)
        add(buttonPanelHolder, BorderLayout.SOUTH);
      else
        add(this.buttonPanel, BorderLayout.SOUTH);
      // re-scan to listen for keys
      listenForKeys(this);

      pack();
    }
  }

  protected ButtonDialog(Frame frame, String title) {
    this(frame, title, null, OK);
  }

  protected ButtonDialog(Frame frame) {
    this(frame, "", null, OK);
  }

  /**
   * defines the set of buttons to be displayed as a set of or'ed button bits
   *  sample:  setButtonSet(ButtonDialog.OK | ButtonDialog.CANCEL | ButtonDialog.HELP)
   *      or:  setButtonSet(ButtonDialog.OK_CANCEL)
   */
  public void setButtonSet(int bs) {
    //System.err.println("ButtonDialog.setButtonSet " + bs);
    if ((bs & ~ALL) != 0)
      throw new IllegalArgumentException(
        Res.format(Res.InvalidButtons, new String[] {Integer.toString(bs & ~ALL)} ));
    if (bs != buttonSet) {
      buttonSet = bs;
      setButtonSet(buttonSetToButtonDescriptors(bs));
    }
  }

  private void setButtonSet(ButtonDescriptor[] buttonDescriptors) {
    this.buttonDescriptors = buttonDescriptors;
    buttonPanel.removeAll();

    for (int i = 0; i < buttonDescriptors.length; i++) {
      Button b = new Button();
      buttonDescriptors[i].button = b;
      b.setActionCommand(buttonDescriptors[i].command);
      b.addActionListener(this);
      buttonPanel.add(b);
    }

    setDefaultLabels();
    setupButtonLabels();
  }

  public int getButtonSet() {
    return buttonSet;
  }

  public void setCentered(boolean c) {
    centered = c;
  }

  public boolean isCentered() {
    return centered;
  }

  public void setEnterOK(boolean e) {
    enterOK = e;
  }
  public boolean isEnterOK() {
    return enterOK;
  }

  public void setEscapeCancel(boolean c) {
    escapeCancel = c;
  }
  public boolean isEscapeCancel() {
    return escapeCancel;
  }

  public void setButtonAlignment(int alignment) {
    if (alignment == buttonAlignment || buttonPanelHolder == null)
      return;

    FlowLayout flow = (FlowLayout) buttonPanelHolder.getLayout();

    if (alignment == FlowLayout.LEFT || alignment == FlowLayout.CENTER || alignment == FlowLayout.RIGHT)
      flow.setAlignment(alignment);
    else
      throw new IllegalArgumentException();

    buttonAlignment = alignment;
    invalidate();
    repaint(100);
  }

  public int getButtonAlignment() {
    return buttonAlignment;
  }

  public synchronized void addActionListener(ActionListener l) {
    //Diagnostic.println("ButtonDialog.addActionListener(" + l.getClass().getName() + ")");
    if (actionMulticaster == null)
      actionMulticaster = new ActionMulticaster();
    actionMulticaster.add(l);
  }

  public synchronized void removeActionListener(ActionListener l) {
    if (actionMulticaster != null)
      actionMulticaster.remove(l);
  }

  protected void processActionEvent(ActionEvent e) {
    //System.err.println("ButtonDialog: processActionEvent " + e);
    result = buttonFromCommand(e.getActionCommand());
    if (actionMulticaster != null) {
      try {
        actionMulticaster.dispatch(e);
      }
      catch (Exception ex) {
        if (result != null && result.closeDialog)
          return;
      }
    }
    // these buttons will close the dialog. listeners can set result to null to abort
    if (result != null && result.closeDialog || e.getActionCommand().equals(CANCEL_COMMAND)) {
      setVisible(false);
      dispose();
    }
  }

  /**
   * set the labels for the buttons actually used, or the whole button set.
   * setting all buttons signifies a label list for the whole set
   */
  public void setLabels(String[] l) {
    if (l != null)
      for (int i = 0; i < labels.length && i < l.length; i++)
        labels[i] = l[i];
    else
      setDefaultLabels();
    setupButtonLabels();
  }

  private void setDefaultLabels() {
    labels  = new String[buttonDescriptors.length];
    for (int i = 0; i < labels.length; ++i)
      labels[i] = buttonDescriptors[i].label;
  }

  protected void setupButtonLabels() {
    if (labels.length == buttonDescriptors.length) {
      for (int i = 0; i < labels.length; i++) // label array is for whole set
        buttonDescriptors[i].button.setLabel(labels[i]);
    }
    else {
      for (int i = 0; i < buttonPanel.getComponentCount(); i++)  // label array is for visible button set
        ((Button)buttonPanel.getComponent(i)).setLabel(labels[i]);
    }
  }

  public String[] getLabels() {
    return labels;
  }

  public void setResult(int id) {
    result = buttonFromID(id);
  }

  public int getResult() {
    return result != null ? result.getID() : NONE;
  }

  public ButtonDescriptor buttonFromID(int id) {
    for (int i = 0; i < buttonDescriptors.length; ++i)
      if (buttonDescriptors[i].id == id)
        return buttonDescriptors[i];
    return null;
  }

  public ButtonDescriptor buttonFromCommand(String actionCommand) {
    for (int i = 0; i < buttonDescriptors.length; ++i)
      if (buttonDescriptors[i].command.equals(actionCommand))
        return buttonDescriptors[i];
    return null;
  }

  public void enableButton(String actionCommand, boolean enableState) {
    ButtonDescriptor bd = buttonFromCommand(actionCommand);
    if (bd != null)
      bd.button.setEnabled(enableState);
  }

  public void show() {
    result = null;

//    pack();

    if (centered)
      centerOnScreen();
    else
      assureOnScreen();
    super.show();
  }

  public void actionPerformed(ActionEvent e) {
    processActionEvent(e);
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
//    if (ps.width < 300)
//      ps.width = 300;
//    if (ps.height < 150)
//      ps.height = 150;
    return ps;
  }

  protected void processWindowEvent(WindowEvent e) {
    //System.err.println("processWindowEvent " + e);
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
      processActionEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, CANCEL_COMMAND));
    else if (e.getID() == WindowEvent.WINDOW_OPENED) {
      // simulate tab from last to wrap around to first & properly set initial focus
      if (getComponentCount() > 0)
        getComponent(getComponentCount()-1).transferFocus();
    }
  }

  protected void centerOnScreen() {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension s = getSize();
    setLocation((d.width - s.width)/2, (d.height - s.height)/2);
  }

  protected void assureOnScreen() {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle b = getBounds();
    if (b == null)
      return;
    if (b.x+b.width > d.width || b.y+b.height > d.height) {
      if (b.x+b.width > d.width)
        b.x -= b.x+b.width - d.width;
      if (b.y+b.height > d.height)
        b.y -= b.y+b.height - d.height;
      setLocation(b.x, b.y);
    }
  }

  protected void dropKeyListeners(Component comp) {
    comp.removeKeyListener(this);
    if (comp instanceof Container) {
      Container c = (Container)comp;
      int count = c.getComponentCount();
      for (int i = 0; i < count; i++)
        dropKeyListeners(c.getComponent(i));
    }
  }

  protected void listenForKeys(Component comp) {
    comp.removeKeyListener(this);
    comp.addKeyListener(this);
    if (comp instanceof Container) {
      Container c = (Container)comp;
      int count = c.getComponentCount();
      for (int i = 0; i < count; i++)
        listenForKeys(c.getComponent(i));
    }
  }

  // KeyListener interface

  /**
   * Process standard dialog accelerators for help,ok,cancel
   */
  public void keyPressed(KeyEvent e) {
    if (e.isConsumed())
      return;
    if (e.getKeyCode() == KeyEvent.VK_ESCAPE && escapeCancel && (e.getModifiers() & (e.SHIFT_MASK | e.CTRL_MASK | e.ALT_MASK)) == 0)
      processActionEvent(new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED, CANCEL_COMMAND));
    else if (e.getKeyCode() == KeyEvent.VK_ENTER && (e.getModifiers() & (e.SHIFT_MASK | e.ALT_MASK)) == 0) {
      boolean isCtrl = (e.getModifiers() & e.CTRL_MASK) != 0;
      boolean needsCtrl = !enterOK || e.getSource() instanceof TextArea;
      if (isCtrl == needsCtrl)
        processActionEvent(new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED, OK_COMMAND));
    }
    else if (e.getKeyCode() == e.VK_F1 && (e.getModifiers() & (e.SHIFT_MASK | e.CTRL_MASK | e.ALT_MASK)) == 0)
      processActionEvent(new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED, HELP_COMMAND));
  }

  /**
   *
   */
  public void keyTyped(KeyEvent e) {
  }

  /**
   *
   */
  public void keyReleased(KeyEvent e) {
  }

}
