/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.control;

import java.awt.*;
import java.awt.event.*;
import com.objectspace.jgl.*;
import borland.jbcl.view.*;
import borland.jbcl.layout.*;

public class ColorChooserPanel
     extends BevelPanel
  implements AdjustmentListener, KeyListener, ItemListener, java.io.Serializable
{
  public static final String CUSTOM_TAG = Res.getString(Res.CustomColorTag);

  public ColorChooserPanel() {
    super();
    setBevelInner(FLAT);
    setMargins(new Insets(0,5,5,5));
    try {
      jbInit();
      hsbMode = true; // make it recalc
      setHsbMode(false);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    value = Color.white;
  }

  public ColorChooserPanel(Color value) {
    this();
    this.value = value;
  }

  ButtonDialog findButtonDialog() {
    Component c = getParent();
    while (c != null && !(c instanceof ButtonDialog))
      c = c.getParent();
    if (c instanceof ButtonDialog)
      return (ButtonDialog)c;
    return null;
  }

  public void addNotify() {
    super.addNotify();
    ButtonDialog bd = findButtonDialog();
    if (bd != null) {
      bd.setEnterOK(true);
      bd.setEscapeCancel(true);
    }
  }

  void jbInit() throws Exception {
    // Standard Color Choice
    Panel choicePan = new Panel();
    choicePan.setLayout(new GridLayout(2,0));
    choicePan.add(new Label(Res.getString(Res.StandardColors)));
    fillChoice();
    choice.addItemListener(this);
    choicePan.add(choice);

    // Custom Aspect Pickers
    Panel pickPan = new Panel();
    pickPan.setLayout(new GridLayout(0,3));
    for (int i = 0; i < 3; i++) {
      pickers[i] = new AspectPicker();
      pickers[i].text.addKeyListener(this);
      pickers[i].scrollbar.addAdjustmentListener(this);
      pickPan.add(pickers[i]);
    }

    // RGB / HSB selection
    Panel checkPan = new Panel();
    CheckboxGroup g = new CheckboxGroup();
    checkPan.add(checkbox[0] = new Checkbox("RGB", false, g));  //NORES
    checkPan.add(checkbox[1] = new Checkbox("HSB", false, g));  //NORES
    checkbox[0].addItemListener(this);
    checkbox[1].addItemListener(this);

    // Custom color groupbox
    GroupBox customPan = new GroupBox(Res.getString(Res.CustomColorGroupBox));
    customPan.setLayout(new BorderLayout());
    customPan.add(pickPan, BorderLayout.CENTER);
    customPan.add(checkPan, BorderLayout.SOUTH);

    // Sample panel
    Panel samplePan = new Panel();
    samplePan.setLayout(new BorderLayout());
    samplePan.add(new Label(Res.getString(Res.Sample)), BorderLayout.NORTH);
    samplePan.add(sample, BorderLayout.CENTER);

    // Split the sample and the custom stuff
    PaneConstraints pc = new PaneConstraints();
    pc.position = PaneConstraints.LEFT;
    SplitPanel split = new SplitPanel();
    split.add(customPan, pc);
    pc.position = PaneConstraints.RIGHT;
    pc.proportion = 0.30f;
    split.add(samplePan, pc);
    split.setGap(2);

    // Now assemble the whole thing...
    this.setLayout(new BorderLayout(0, 5));
    this.add(choicePan, BorderLayout.NORTH);
    this.add(split, BorderLayout.CENTER);
  }

  public Color getColorValue() {
    return value;
  }

  public void setColorValue(Color value) {
    changeColor(value);
    changed = false;
  }

  public boolean isChanged() {
    return changed;
  }

  protected void colorChanged(Color newColor) {
    changed = true;
  }

  protected void changeColor(Color value) {
    changeColor(value, false, false);
  }

  protected void changeColor(Color value, boolean suppressText, boolean suppressScroll) {
    if (value != null && value.equals(this.value))
      return;
    this.value = value;
    if (value == null)
      return;

    // decompose the color
    if (!hsbMode) {
      comp[0] = value.getRed();
      comp[1] = value.getGreen();
      comp[2] = value.getBlue();
    }
    else {
      float[] hsbComp = Color.RGBtoHSB(value.getRed(), value.getGreen(), value.getBlue(), null);
      comp[0] = (int)(hsbComp[0]*100);
      comp[1] = (int)(hsbComp[1]*100);
      comp[2] = (int)(hsbComp[2]*100);
    }

    //System.err.println("changeColor: " + value + " hsbMode:" + hsbMode + " 0:" + comp[0] + " 1:" + comp[1] + " 2:" + comp[2]);

    // update text columns and scrollers with color components--rgb or hsb
    for (int i = 0; i < 3; i++) {
      if (!suppressText) {
        String t = Integer.toString(comp[i]);
        if (text[i] == null || !text[i].equals(t)) {
          text[i] = t;
          pickers[i].text.setText(t);
        }
      }
      int max = hsbMode ? 100 : 255;
      if (!suppressScroll && pickers[i].scrollbar.getValue() != comp[i])
        pickers[i].scrollbar.setValues(comp[i], 0, 0, max+1);
    }

    String text = valueToText(value, true);
    if (text != null)
      choice.select(text);

    if (value != sample.getForeground())
      sample.setForeground(value);

    if (!hsbMode) {
      pickers[0].sample.setForeground(new Color(255, 255 - comp[0], 255 - comp[0]));
      pickers[1].sample.setForeground(new Color(255 - comp[1], 255, 255 - comp[1]));
      pickers[2].sample.setForeground(new Color(255 - comp[2], 255 - comp[2], 255));
    }
//    else {
//      pickers[0].sample.setForeground(Color.getHSBColor(comp[0], 50, 100));
//      pickers[1].sample.setForeground(Color.getHSBColor(comp[0], comp[1], 100));
//      pickers[2].sample.setForeground(Color.getHSBColor(comp[0], 50, comp[2]));
//    }

    colorChanged(value);
  }

  public void setHsbMode(boolean hsbMode) {
    if (this.hsbMode != hsbMode) {
      this.hsbMode = hsbMode;
      int max = hsbMode ? 100 : 255;
      for (int i = 0; i < 3; i++) {
        pickers[i].label.setText(hsbMode ? hsbLabels[i] : rgbLabels[i]);
        pickers[i].scrollbar.setValues(comp[i], 0, 0, max+1);
        pickers[i].sample.setVisible(!hsbMode);
      }
      checkbox[0].setState(!hsbMode);
      checkbox[1].setState(hsbMode);
      Color newValue = value;
      value = null;
      changeColor(newValue);
    }
  }

  public boolean isHsbMode() {
    return hsbMode;
  }

  public void adjustmentValueChanged(AdjustmentEvent event) {
    Color c;
    if (hsbMode)
      c = Color.getHSBColor(Math.min(pickers[0].scrollbar.getValue() / 100.0f, 0.999999f),
                                     pickers[1].scrollbar.getValue() / 100.0f,
                                     pickers[2].scrollbar.getValue() / 100.0f);
    else
      c = new Color(pickers[0].scrollbar.getValue(),
                    pickers[1].scrollbar.getValue(),
                    pickers[2].scrollbar.getValue());
    if (!c.equals(value))
      changeColor(c, false, false);
  }

  public void keyTyped(KeyEvent e) {}
  public void keyPressed(KeyEvent e) {}
  public void keyReleased(KeyEvent e) {
    try {
      int[] t = new int[3];
      boolean bad = false;
      int max = hsbMode ? 100 : 255;
      for (int i = 0; i < 3; i++) {
        t[i] = Integer.parseInt(pickers[i].text.getText());
        if (t[i] < 0 || t[i] > max)
          bad = true;
      }
      if (!bad) {
        Color c;
        if (hsbMode)
          c = Color.getHSBColor(Math.min(t[0] / 100.0f, 0.999999f), t[1] / 100.0f, t[2] / 100.0f);
        else
          c = new Color(t[0], t[1], t[2]);
        if (!c.equals(value))
          changeColor(c, true, false);
      }
    }
    catch (NumberFormatException x) {
    }
  }

  public void itemStateChanged(ItemEvent e) {
    if (e.getSource() == choice) {
      Color c = textToValue(choice.getSelectedItem(), true);
      if (c != null)
        changeColor(c, false, false);
    }
    else {
    //System.err.println("itemStateChanged: " + e);
//    setHsbMode(e.getStateChange() == ItemEvent.SELECTED);
      setHsbMode(e.getItemSelectable() == checkbox[1]);
    }
  }

  void fillChoice() {
    for (int i = 0; i < colorValues.length; i++)
      choice.add(colorNames[i][0]);
  }

  // colorNames[0] = display text (translated)
  // colorNames[1] = source code text (not translated)
  //
  static final String[][] colorNames = {
    {Res.getString(Res.Color_white),                 "Color.white"},                       // NORES
    {Res.getString(Res.Color_lightGray),             "Color.lightGray"},                   // NORES
    {Res.getString(Res.Color_gray),                  "Color.gray"},                        // NORES
    {Res.getString(Res.Color_darkGray),              "Color.darkGray"},                    // NORES
    {Res.getString(Res.Color_black),                 "Color.black"},                       // NORES
    {Res.getString(Res.Color_red),                   "Color.red"},                         // NORES
    {Res.getString(Res.Color_pink),                  "Color.pink"},                        // NORES
    {Res.getString(Res.Color_orange),                "Color.orange"},                      // NORES
    {Res.getString(Res.Color_yellow),                "Color.yellow"},                      // NORES
    {Res.getString(Res.Color_green),                 "Color.green"},                       // NORES
    {Res.getString(Res.Color_magenta),               "Color.magenta"},                     // NORES
    {Res.getString(Res.Color_cyan),                  "Color.cyan"},                        // NORES
    {Res.getString(Res.Color_blue),                  "Color.blue"},                        // NORES
    {Res.getString(Res.Color_desktop),               "SystemColor.desktop"},               // NORES
    {Res.getString(Res.Color_activeCaption),         "SystemColor.activeCaption"},         // NORES
    {Res.getString(Res.Color_activeCaptionText),     "SystemColor.activeCaptionText"},     // NORES
    {Res.getString(Res.Color_activeCaptionBorder),   "SystemColor.activeCaptionBorder"},   // NORES
    {Res.getString(Res.Color_inactiveCaption),       "SystemColor.inactiveCaption"},       // NORES
    {Res.getString(Res.Color_inactiveCaptionText),   "SystemColor.inactiveCaptionText"},   // NORES
    {Res.getString(Res.Color_inactiveCaptionBorder), "SystemColor.inactiveCaptionBorder"}, // NORES
    {Res.getString(Res.Color_window),                "SystemColor.window"},                // NORES
    {Res.getString(Res.Color_windowBorder),          "SystemColor.windowBorder"},          // NORES
    {Res.getString(Res.Color_windowText),            "SystemColor.windowText"},            // NORES
    {Res.getString(Res.Color_menu),                  "SystemColor.menu"},                  // NORES
    {Res.getString(Res.Color_menuText),              "SystemColor.menuText"},              // NORES
    {Res.getString(Res.Color_text),                  "SystemColor.text"},                  // NORES
    {Res.getString(Res.Color_textText),              "SystemColor.textText"},              // NORES
    {Res.getString(Res.Color_textHighlight),         "SystemColor.textHighlight"},         // NORES
    {Res.getString(Res.Color_textHighlightText),     "SystemColor.textHighlightText"},     // NORES
    {Res.getString(Res.Color_textInactiveText),      "SystemColor.textInactiveText"},      // NORES
    {Res.getString(Res.Color_control),               "SystemColor.control"},               // NORES
    {Res.getString(Res.Color_controlText),           "SystemColor.controlText"},           // NORES
    {Res.getString(Res.Color_controlHighlight),      "SystemColor.controlHighlight"},      // NORES
    {Res.getString(Res.Color_controlLtHighlight),    "SystemColor.controlLtHighlight"},    // NORES
    {Res.getString(Res.Color_controlShadow),         "SystemColor.controlShadow"},         // NORES
    {Res.getString(Res.Color_controlDkShadow),       "SystemColor.controlDkShadow"},       // NORES
    {Res.getString(Res.Color_scrollbar),             "SystemColor.scrollbar"},             // NORES
    {Res.getString(Res.Color_info),                  "SystemColor.info"},                  // NORES
    {Res.getString(Res.Color_infoText),              "SystemColor.infoText"},              // NORES
    {CUSTOM_TAG, CUSTOM_TAG},
  };

  static final Color[] colorValues = {
    Color.white,
    Color.lightGray,
    Color.gray,
    Color.darkGray,
    Color.black,
    Color.red,
    Color.pink,
    Color.orange,
    Color.yellow,
    Color.green,
    Color.magenta,
    Color.cyan,
    Color.blue,
    SystemColor.desktop,
    SystemColor.activeCaption,
    SystemColor.activeCaptionText,
    SystemColor.activeCaptionBorder,
    SystemColor.inactiveCaption,
    SystemColor.inactiveCaptionText,
    SystemColor.inactiveCaptionBorder,
    SystemColor.window,
    SystemColor.windowBorder,
    SystemColor.windowText,
    SystemColor.menu,
    SystemColor.menuText,
    SystemColor.text,
    SystemColor.textText,
    SystemColor.textHighlight,
    SystemColor.textHighlightText,
    SystemColor.textInactiveText,
    SystemColor.control,
    SystemColor.controlText,
    SystemColor.controlHighlight,
    SystemColor.controlLtHighlight,
    SystemColor.controlShadow,
    SystemColor.controlDkShadow,
    SystemColor.scrollbar,
    SystemColor.info,
    SystemColor.infoText,
    null,
  };

  public static String valueToText(Color c, boolean localized) {
    for (int i = 0; i < colorValues.length; i++)
      if (c == colorValues[i])
        return localized ? colorNames[i][0] : colorNames[i][1];
    for (int i = 0; i < colorValues.length; i++)
      if (c.equals(colorValues[i]))
        return localized ? colorNames[i][0] : colorNames[i][1];
    return CUSTOM_TAG;
  }

  public static Color textToValue(String text, boolean localized) {
    return localized ? (Color)localizedTextToValueMap.get(text) : (Color)textToValueMap.get(text);
  }

  static HashMap textToValueMap;
  static HashMap localizedTextToValueMap;

  static {
    int i = 0;
    try {
      textToValueMap = new HashMap();
      localizedTextToValueMap = new HashMap();
      //System.err.println("color values len -" +colorValues.length);
      for ( i = 0; i < colorValues.length; i++) {
        if (colorValues[i] != null) {
          localizedTextToValueMap.put(colorNames[i][0], colorValues[i]);
          textToValueMap.put(colorNames[i][1], colorValues[i]);
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
      System.err.println("color values len " + i + " " + colorValues[i] + colorNames[i][0] + colorNames[i][1]);
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    // widen labels (for Hue/Saturation/Brightness)
    ps.width += 40;
    return ps;
  }

  private Color          value;
  private ShapeControl   sample    = new ShapeControl(ShapeControl.ROUND_RECT);
  private Checkbox[]     checkbox  = new Checkbox[2];
  private String[]       rgbLabels = {Res.getString(Res.Red), Res.getString(Res.Green), Res.getString(Res.Blue)};
  private String[]       hsbLabels = {Res.getString(Res.Hue), Res.getString(Res.Saturation), Res.getString(Res.Brightness)};
  private int[]          comp      = new int[3];
  private boolean        hsbMode   = false;
  private boolean        changed   = false;
  private String[]       text      = new String[3];
  private Choice         choice    = new Choice();
  private AspectPicker[] pickers   = new AspectPicker[3];
}

class AspectPicker extends java.awt.Container implements java.io.Serializable
{
  public AspectPicker() {
    sample.setDrawEdge(false);
    bevel.setBevelInner(BevelPanel.LOWERED);
    bevel.setBevelOuter(BevelPanel.LOWERED);
    bevel.setLayout(new BorderLayout());
    pan1.setLayout(new BorderLayout());
    pan1.add(text, BorderLayout.NORTH);
    pan1.add(bevel, BorderLayout.CENTER);
    bevel.add(sample, BorderLayout.CENTER);
    bevel.add(scrollbar, BorderLayout.NORTH);
    this.setLayout(new BorderLayout());
    this.add(label, BorderLayout.NORTH);
    this.add(pan1, BorderLayout.CENTER);
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    // shrink up ShapeControl
    ps.width -= 80;
    ps.height -= 80;
    return ps;
  }

  ShapeControl sample    = new ShapeControl();   // color level sample (for this aspect)
  Scrollbar    scrollbar = new Scrollbar(Scrollbar.HORIZONTAL); // draggable value changes
  TextField    text      = new TextField("", 3); // numeric setting
  Label        label     = new Label();          // "Red:" or "Hue:" etc...
  BevelPanel   bevel     = new BevelPanel();     // layout containment panel
  Panel        pan1      = new Panel();          // layout containment panel
}
