/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.control;

import java.awt.*;
import java.awt.event.*;
import borland.jbcl.layout.*;
import borland.jbcl.util.Diagnostic;
import com.sun.java.swing.*;

public class FontChooserPanel extends JComponent implements ItemListener, KeyListener, java.io.Serializable
{
  public FontChooserPanel() {
    try {
      init();
    }
    catch (Exception x) {
      Diagnostic.printStackTrace(x);
    }
  }

  public FontChooserPanel(Font font) {
    this();
    setFontValue(font);
  }

  ButtonDialog findButtonDialog() {
    Component c = getParent();
    while (c != null && !(c instanceof ButtonDialog))
      c = c.getParent();
    if (c instanceof ButtonDialog)
      return (ButtonDialog)c;
    return null;
  }

  public void addNotify() {
    super.addNotify();
    ButtonDialog bd = findButtonDialog();
    if (bd != null) {
      bd.setEnterOK(true);
      bd.setEscapeCancel(true);
    }
  }

  Font font;
  boolean changed;
  XYLayout xYLayout1 = new XYLayout();
  XYLayout xYLayout2 = new XYLayout();
  GroupBox groupBox1 = new GroupBox();
  List nameList = new List();
  TextField sizeField = new TextField();
  Label previewField = new Label();
  Checkbox boldBox = new Checkbox();
  Checkbox italicBox = new Checkbox();

  void init() {

    this.setLayout(xYLayout1);

    boldBox.setLabel(Res.getString(Res.Bold));
    italicBox.setLabel(Res.getString(Res.Italic));
    groupBox1.setLabel(Res.getString(Res.Size));
    previewField.setText(Res.getString(Res.FontSample));

    this.add(nameList, new XYConstraints(13, 13, 158, 98));
    this.add(boldBox, new XYConstraints(190, 14, 115, 20));
    this.add(italicBox, new XYConstraints(190, 37, 115, 20));
    this.add(groupBox1, new XYConstraints(182, 61, 110, 55));
    groupBox1.setLayout(new BorderLayout());
    groupBox1.add(sizeField);
    this.add(previewField, new XYConstraints(11, 125, 278, 45));

    Toolkit toolkit = Toolkit.getDefaultToolkit();
    String[] fontList = toolkit.getFontList();
    for (int i = 0; i < fontList.length; i++)
      nameList.addItem(fontList[i]);

    nameList.addItemListener(this);
    boldBox.addItemListener(this);
    italicBox.addItemListener(this);
    sizeField.addKeyListener(this);
  }

  public Font getFontValue() {
    return font;
  }

  public void setFontValue(Font font) {
    if (font != null) {
      sizeField.setText(Integer.toString(font.getSize()));
      changeFont(font);
    }
    else {
      sizeField.setText("12");
      nameList.select(-1);
    }
    sizeField.setEnabled(font != null);
    boldBox.setEnabled(font != null);
    italicBox.setEnabled(font != null);
    changed = false;
  }

  public boolean isChanged() {
    return changed;
  }

  protected void fontChanged(Font newFont) {
    changed = true;
  }

  protected void changeFont(Font font) {
    changeFont(font, false, false);
  }

  protected void changeFont(Font font, boolean suppressList, boolean suppressText) {
    //System.err.println("changeFont: " + this.font + " to: " + font);
    if (font != null && font.equals(this.font))
      return;
    this.font = font;
    if (font == null)
      return;

    Toolkit toolkit = Toolkit.getDefaultToolkit();
    String[] fontList = toolkit.getFontList();

    if (!suppressList && fontList != null && nameList != null) {
      for (int i = 0; i < fontList.length; i++)
        if (fontList[i].equals(font.getName())) {
          nameList.select(i);
          break;
        }
    }

    /*
    if (!suppressText && sizeField != null)
      sizeField.setText(Integer.toString(font.getSize()));
    */
    if (boldBox != null)
      boldBox.setState(font.isBold());
    if (italicBox != null)
      italicBox.setState(font.isItalic());

    previewField.setFont(font);
    //System.err.println("FONT:"+font);
    fontChanged(font);
  }

  int getFontSize() {
    try {
      int sz = Integer.parseInt(sizeField.getText());
      if (sz > 30000) {
        sizeField.setText(String.valueOf(30000));
      }
      return sz < 30000 ? sz : 30000;
    }
    catch (NumberFormatException e) {
    }
    return 1;
  }

  public void itemStateChanged(ItemEvent e) {
    String name = nameList.getSelectedItem();
    sizeField.setEnabled(true);
    boldBox.setEnabled(true);
    italicBox.setEnabled(true);
    Font f = new Font(name != null ? name : "Dialog",  //NORES
                      (boldBox.getState() ? Font.BOLD : Font.PLAIN) +
                      (italicBox.getState() ? Font.ITALIC : Font.PLAIN),
                      getFontSize());
//System.err.println("FontChooserPanel.itemStateChanged(" + e + ") => " + f);
    changeFont(f, e != null && e.getSource() instanceof List, false);
  }

  public void keyPressed(KeyEvent e) {
  }
  public void keyTyped(KeyEvent e) {
  }
  public void keyReleased(KeyEvent e) {
    itemStateChanged(null);
  }

  public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    d.width += 8;
    d.height += 8;
    return d;
  }
}
