/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.control;

import java.awt.*;
import java.awt.event.*;
import borland.jbcl.util.*;
import com.objectspace.jgl.Array;

public class Message implements WindowListener, java.io.Serializable
{
  public static final int OK     = ButtonDialog.OK;
  public static final int YES    = ButtonDialog.YES;
  public static final int NO     = ButtonDialog.NO;
  public static final int CANCEL = ButtonDialog.CANCEL;

  public static final int OK_CANCEL     = ButtonDialog.OK_CANCEL;
  public static final int YES_NO        = ButtonDialog.YES_NO;
  public static final int YES_NO_CANCEL = ButtonDialog.YES_NO_CANCEL;

  public Message(Frame frame, String title, String msg) {
    this.frame = frame;
    this.title = title;
    this.message = msg;
    if (frame != null) {
      dialog = new MessageDialog(frame, title, msg);
      dialog.addWindowListener(this);
    }
  }

  public Message(Frame frame, String title) {
    this(frame, title, "");
  }

  public Message(Frame frame) {
    this(frame, "", "");
  }

  public Message() {
    this(null, "", "");
  }

  public void setFrame(Frame frame) {
    this.frame = frame;
  }

  public Frame getFrame() {
    return frame;
  }

  public void setTitle(String title) {
    this.title = title;
    if (dialog != null)
      dialog.setTitle(title);
  }

  public String getTitle() {
    if (dialog != null)
      title = dialog.getTitle();
    return title;
  }

  public void setMessage(String message) {
    this.message = message;
    if (dialog != null)
      dialog.setMessage(message);
  }

  public String getMessage() {
    if (dialog != null)
      message = dialog.getMessage();
    return message;
  }

  public void setButtonSet(int bs) {
    if (bs != buttonSet) {
      buttonSet = bs;
      if (dialog != null)
        dialog.setButtonSet(bs);
    }
  }

  public int getButtonSet() {
    return buttonSet;
  }

  public void setLabels(String[] l) {
    labels = new String[l.length];
    System.arraycopy(l, 0, labels, 0, l.length);
    if (dialog != null)
      dialog.setLabels(l);
  }

  public String[] getLabels() {
    if (dialog != null)
      labels = dialog.getLabels();
    return labels;
  }

  public void setResult(int i) {
    result = i;
    if (dialog != null)
      dialog.setResult(result);
  }

  public int getResult() {
    if (dialog != null)
      result = dialog.getResult();
    return result;
  }

  public void show() {
    setVisible(true);
  }

  public void setVisible(boolean visible) {
    if (visible) {
      if (dialog == null) {
        if (frame == null)
          throw new IllegalStateException(Res.getString(Res.NoFrame));
        dialog = new MessageDialog(frame, title, message);
        if (buttonSet >= 0)
          dialog.setButtonSet(buttonSet);
        if (labels != null)
          dialog.setLabels(labels);
        dialog.setResult(result);
        dialog.addWindowListener(this);
        dialog.addActionListener(actionMulticaster);
      }
      dialog.pack();
      dialog.show();
    }
    else if (dialog != null) {
      dialog.setVisible(false);
    }
  }

  public boolean isVisible() {
    return dialog != null ? dialog.isVisible() : false;
  }

  public void addActionListener(ActionListener l) {
    actionMulticaster.add(l);
  }

  public void removeActionListener(ActionListener l) {
    actionMulticaster.remove(l);
  }

  public void windowOpened(WindowEvent e) {}
  public void windowClosing(WindowEvent e) {
    result = dialog.getResult();
  }
  public void windowClosed(WindowEvent e) {
    if (dialog != null)
      result = dialog.getResult();
    dialog = null;
    if (frame.isShowing()) {
      if (frame.getFocusOwner() != null)
        frame.getFocusOwner().requestFocus();
      else
        frame.requestFocus();
    }
  }
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}

  protected MessageDialog dialog;
  protected Frame frame;
  protected String title;
  protected String message;
  protected int buttonSet = MessageDialog.OK;
  protected String[] labels;
  protected int result;
  protected transient ActionMulticaster actionMulticaster = new ActionMulticaster();
}
