/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//NOTTRANSLATABLE
package borland.jbcl.control.applet;

import java.util.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.*;
import java.applet.*;
import java.awt.image.MemoryImageSource;
import java.awt.image.ColorModel;

import sun.misc.MessageUtils;

import borland.jbcl.util.Diagnostic;

/**
 * Applet panel class. The panel manages and manipulates the
 * applet as it is being loaded. It forks a separate thread in a new
 * thread group to call the applet's init(), start(), stop(), and
 * destroy() methods.
 *
 * @version     1.40, 01/27/97
 * @author      Arthur van Hoff
 */
public abstract class AppletPanel extends Panel implements AppletStub, Runnable {
    /* Are we debugging? */
    static boolean debug = false;

    /**
     * The applet (if loaded).
     */
    Applet applet;

    /**
     * Applet will allow initialization
     */
    boolean doInit;


    /**
     * The classloader for the applet.
     */
    public AppletClassLoader loader;

    /**
     * The current status. One of: APPLET_DISPOSE, APPLET_LOAD,
     * APPLET_INIT, APPLET_START, APPLET_STOP,
     * APPLET_DESTROY, APPLET_ERROR.
     */
    protected int status;

    /**
     * The thread for the applet.
     */
    Thread handler;

    /* applet event ids */
    public final static int APPLET_DISPOSE = 0;
    public final static int APPLET_LOAD = 1;
    public final static int APPLET_INIT = 2;
    public final static int APPLET_START = 3;
    public final static int APPLET_STOP = 4;
    public final static int APPLET_DESTROY = 5;
    public final static int APPLET_QUIT = 6;
    public final static int APPLET_ERROR = 7;

    /* send to the parent to force relayout */
    public final static int APPLET_RESIZE = 51234;

    /* sent to a (distant) parent to indicate that the applet is being
     * loaded or as completed loading
     */
    public final static int APPLET_LOADING = 51235;
    public final static int APPLET_LOADING_COMPLETED = 51236;

    /**
     * The applet size.
     */
    public Dimension appletSize = new Dimension(100, 100);

    MessageUtils mu = new MessageUtils();

    /**
     * The thread to use during applet loading
     */

    Thread loaderThread = null;

    /**
     * Flag to indicate that a loading has been cancelled
     */
    boolean loadAbortRequest = false;

    /* abstract classes */
    abstract protected String getCode();
    abstract protected int    getWidth();
    abstract protected int    getHeight();
    abstract protected String getJarFiles();
    abstract protected String getSerializedObject();

    /**
     * Construct an applet viewer and start the applet.
     */
    public void init() {
        setLayout(new BorderLayout());

        try {
            // Get the width (if any)
            appletSize.width = getWidth();

            // Get the height (if any)
            appletSize.height = getHeight();

        } catch (NumberFormatException e) {
            // Turn on the error flag and let TagAppletPanel
            // do the right thing.
            status = APPLET_ERROR;
            showAppletStatus("badattribute.exception");
            showAppletLog("badattribute.exception");
            showAppletException(e);
        }

        // Create a thread group for the applet, and start a new
        // thread to load the applet.
        String nm = "applet-" + getCode();
        loader = getClassLoader(getCodeBase());
        ThreadGroup appletGroup = loader.getThreadGroup();
        handler = new Thread(appletGroup, this, "thread " + nm);



//      handler.start();
    }

    /**
     * Minimum size
     */
    public Dimension minimumSize() {
        return new Dimension(appletSize.width, appletSize.height);
    }

    /**
     * Preferred size
     */
    public Dimension preferredSize() {
        return minimumSize();
    }

    /**
     * Event Queue
     */
    Event queue = null;

    /**
     * Send an event.
     */
    public void sendEvent(int id) {
        //System.err.println("SEND= " + id);
        sendEvent(new Event(null, id, null));
    }

    /**
     * Send an event. Queue it for execution by the handler thread.
     */
    protected synchronized void sendEvent(Event evt) {
        if (queue == null) {
            //System.err.println("SEND0= " + evt);
            evt.target = queue;
            queue = evt;
            notifyAll();
        } else {
            //System.err.println("SEND1= " + evt);
            Event q = queue;
            for (; q.target != null ; q = (Event)q.target);
            q.target = evt;
        }
    }

    /**
     * Get an event from the queue.
     */
    synchronized Event getNextEvent() throws InterruptedException {
        while (queue == null) {
            wait();
        }
        Event evt = queue;
        queue = (Event)queue.target;
        evt.target = this;
        return evt;
    }

    /**
     * Execute applet events.
     */
    public void run() {

        Thread curThread = Thread.currentThread();
        if (curThread == loaderThread) {
            // if we are in the loader thread, cause
            // loading to occur.  We may exit this with
            // status being APPLET_DISPOSE, APPLET_ERROR,
            // or APPLET_LOAD
            runLoader();
            return;
        }

        while (true) {
            Event evt;
            try {
                evt = getNextEvent();


            } catch (InterruptedException e) {
                showAppletStatus("bail");
                return;
            }

//System.err.println("\nEVENT.id = " + evt.id);
//System.err.println("EVENT = " + evt);

            //showAppletStatus("EVENT = " + evt.id);
            try {
                switch (evt.id) {

                  case APPLET_LOAD:
                    if (!okToLoad()) {
                        break;
                    }
                    // This complexity allows laoding of applets to be
                    // interruptable.  The actual thread loading runs
                    // in a separate thread, so it can be interrupted
                    // without harming the applet thread.
                    // So that we don't have to worry about
                    // concurrency issues, the main applet thread waits
                    // until the loader thread terminates.
                    // (one way or another).
                    if (loaderThread == null) {
                        // REMIND: do we want a name?
                        //System.err.println("------------------- loading applet");
                        setLoaderThread(new Thread(this));
                        loaderThread.start();
                        // we get to go to sleep while this runs
                        loaderThread.join();
                        setLoaderThread(null);
                    } else {
                        // REMIND: issue an error -- this case should never
                        // occur.
                    }
                    break;



                  case APPLET_INIT:
                    if (status != APPLET_LOAD) {
                        showAppletStatus("notloaded");
                        break;
                    }
                    applet.setSize(appletSize);
                    try {
                        if (doInit) {
                            applet.init();
                        }
                    } catch (ThreadDeath e) {
                        System.err.println("**ThreadDeath initing applet: " + e);
                        getAppletContext().showStatus("NOOP");
                        break;
                    } catch (Error e) {
                        System.err.println("**Error initing applet: " + e);
                        getAppletContext().showStatus("NOOP");
                        break;
                    } catch (Exception e) {
                        System.err.println("**Exception initing applet: " + e);
                        getAppletContext().showStatus("NOOP");
                        break;
                    }
                    doInit = true;      // allow restarts
                    validate();
                    status = APPLET_INIT;
                    showAppletStatus("inited");
                    getAppletContext().showStatus("OK");
                    break;

                  case APPLET_START:
                    if (status != APPLET_INIT) {
                        showAppletStatus("notinited");
                        break;
                    }
                    applet.setSize(appletSize);
                    applet.start();
                    validate();
                    applet.setVisible(true);
                    status = APPLET_START;
                    showAppletStatus("started");
                    break;

                  case APPLET_STOP:
                    if (status != APPLET_START) {
                        showAppletStatus("notstarted");
                        break;
                    }
                    status = APPLET_INIT;
//                  applet.setVisible(false);
                    applet.stop();
                    showAppletStatus("stopped");
                    break;

                  case APPLET_DESTROY:
                    if (status != APPLET_INIT) {
                        showAppletStatus("notstopped");
                        break;
                    }
                    status = APPLET_LOAD;
                    applet.destroy();
                    showAppletStatus("destroyed");
                    break;

                  case APPLET_DISPOSE:
                    if (status != APPLET_LOAD) {
                        showAppletStatus("notdestroyed");
                        break;
                    }
                    status = APPLET_DISPOSE;
                    remove(applet);
                    applet = null;
                    showAppletStatus("disposed");
                    break;


                  case APPLET_QUIT:
                    return;
                }
            } catch (ThreadDeath e) {
                //System.err.println("**ThreadDeath in AppletPanel run: " + e);
                showAppletStatus("death");
                return;
            } catch (Error e) {
                //System.err.println("**Error in AppletPanel run: " + e);
                showAppletStatus("error2", e.getClass().getName(), e.getMessage());
                showAppletException(e);
            } catch (Exception e) {
                //System.err.println("**Exception in AppletPanel run: " + e);
                showAppletStatus("exception2", e.getClass().getName(),
                                                         e.getMessage());
                showAppletException(e);
            }

            clearLoadAbortRequest();

        }
    }


    /**
     * Load the applet into memory.
     * Runs in a seperate (and interruptible) thread from the rest of the
     * applet event processing so that it can be gracefully interrupted from
     * things like HotJava.
     */
    private void runLoader() {
        if (status != APPLET_DISPOSE) {
            showAppletStatus("notdisposed");
            return;
        }

        postEvent(new Event(this, APPLET_LOADING, null));
//        dispatchEvent(AWTEvent e);

        // REMIND -- might be cool to visually indicate loading here --
        // maybe do animation?
        status = APPLET_LOAD;

        // Create a class loader
        loader = getClassLoader(getCodeBase());

        // Load the archives if present.
        // REMIND - this probably should be done in a separate thread,
        // or at least the additional archives (epll).

        debug("AppletPanel:CTOR: get list of JarFiles");
        String jarFiles = getJarFiles();

        if (jarFiles == null) {
            debug("No jar files specified");
        } else {
            debug("archive parameter is \"" + jarFiles +"\"");

            AppletResourceLoader resourceLoader = loader.getResourceLoader();
            StringTokenizer st = new StringTokenizer(jarFiles, ",", false);
            while(st.hasMoreTokens()) {
                String tok = st.nextToken().trim();
                debug("load archive for codebase " + getCodeBase() +
                      ", file " + tok);
                try {
                    resourceLoader.loadJar(getCodeBase(), tok);
                } catch (IOException e) {
                    debug("error loading archive " + tok, e);
                }
            }
        }

        // Create the applet using the class loader
        String serName = getSerializedObject();
        String code = getCode();

        if (code != null && serName != null) {
            System.err.println(amh.getMessage("runloader.err"));
            return;
        }
        if (code == null && serName == null) {
            String msg = "nocode";
            status = APPLET_ERROR;
            showAppletStatus(msg);
            showAppletLog(msg);
            repaint();
        }
        try {
            if (code != null) {
                applet = (Applet)loader.loadCode(code).newInstance();
                doInit = true;
            } else {
                // serName is not null;
                try {
                    InputStream is = loader.getResourceAsStream(serName);
                    ObjectInputStream ois =
                        new AppletObjectInputStream(is, loader);
                    Object serObject = ois.readObject();
                    applet = (Applet) serObject;
                    doInit = false; // skip over the first init
                } catch (Exception ex) {
                    System.err.println(amh.getMessage("runloader.exception", serName));

                    Diagnostic.printStackTrace(ex);
                    // something more intelligent.
                    return;
                }
            }
            if (Thread.interrupted()) {
                try {
                    status = APPLET_DISPOSE; // APPLET_ERROR?
                    applet = null;
                    // REMIND: This may not be exactly the right thing: the
                    // status is set by the stop button and not necessarily
                    // here.
                    showAppletStatus("death");
                } finally {
                    Thread.currentThread().interrupt(); // resignal interrupt
                }
                return;
            }
        } catch (ClassNotFoundException e) {
            status = APPLET_ERROR;
            showAppletStatus("notfound", code);
            showAppletLog("notfound", code);
            showAppletException(e);
            return;
        } catch (InstantiationException e) {
            status = APPLET_ERROR;
            showAppletStatus("nocreate", code);
            showAppletLog("nocreate", code);
            showAppletException(e);
            return;
        } catch (IllegalAccessException e) {
            status = APPLET_ERROR;
            showAppletStatus("noconstruct", code);
            showAppletLog("noconstruct", code);
            showAppletException(e);
            // sbb -- I added a return here
            return;
        } catch (Exception e) {
            status = APPLET_ERROR;
            showAppletStatus("exception", e.getMessage());
            showAppletException(e);
            return;
        } catch (ThreadDeath e) {
            status = APPLET_ERROR;
            showAppletStatus("death");
            return;
        } catch (Error e) {
            status = APPLET_ERROR;
            showAppletStatus("error", e.getMessage());
            showAppletException(e);
            return;
        } finally {
            // notify that loading is no longer going on
            postEvent(new Event(this, APPLET_LOADING_COMPLETED, null));
//            dispatchEvent(AWTEvent e);
        }

        // Stick it in the frame
        applet.setStub(this);
        applet.setVisible(false);
        add("Center", applet);
        showAppletStatus("loaded");
        validate();
    }


    /**
     * Request that the loading of the applet be stopped.
     */
    protected synchronized void stopLoading() {
        // REMIND: fill in the body
        if (loaderThread != null) {
            //System.err.println("Interrupting applet loader thread: " + loaderThread);
            loaderThread.interrupt();
        } else {
            setLoadAbortRequest();
        }
    }


    protected synchronized boolean okToLoad() {
        return !loadAbortRequest;
    }

    protected synchronized void clearLoadAbortRequest() {
        loadAbortRequest = false;
    }

    protected synchronized void setLoadAbortRequest() {
        loadAbortRequest = true;
    }


    private synchronized void setLoaderThread(Thread loaderThread) {
        this.loaderThread = loaderThread;
    }

    /**
     * Return true when the applet has been started.
     */
    public boolean isActive() {
        return status == APPLET_START;
    }

    /**
     * Is called when the applet wants to be resized.
     */
    public void appletResize(int width, int height) {
        appletSize.width = width;
        appletSize.height = height;
        postEvent(new Event(this, APPLET_RESIZE, preferredSize()));
//        dispatchEvent(AWTEvent e);
    }

    public Applet getApplet() {
        return applet;
    }

    /**
     * Status line. Called by the AppletPanel to provide
     * feedback on the Applet's state.
     */
    protected void showAppletStatus(String status) {
        getAppletContext().showStatus(amh.getMessage(status));
    }

    protected void showAppletStatus(String status, Object arg) {
        getAppletContext().showStatus(amh.getMessage(status, arg));
    }
    protected void showAppletStatus(String status, Object arg1, Object arg2) {
        getAppletContext().showStatus(amh.getMessage(status, arg1, arg2));
    }

    /**
     * Called by the AppletPanel to print to the log.
     */
    protected void showAppletLog(String msg) {
        System.err.println(amh.getMessage(msg));
    }

    protected void showAppletLog(String msg, Object arg) {
        System.err.println(amh.getMessage(msg, arg));
    }

    /**
     * Called by the AppletPanel to provide
     * feedback when an exception has happened.
     */
    protected void showAppletException(Throwable t) {
        t.printStackTrace();
        repaint();
    }

    /**
     * The class loaders
    */
    private static Hashtable classloaders = new Hashtable();

    /**
     * Flush a class loader.
     */
    public static synchronized void flushClassLoader(URL codebase) {
        classloaders.remove(codebase);
    }

    /**
     * Flush all class loaders.
     */
    public static synchronized void flushClassLoaders() {
        classloaders = new Hashtable();
    }

    /**
     * Get a class loader.
     */
    synchronized AppletClassLoader getClassLoader(URL codebase) {
        AppletClassLoader c = (AppletClassLoader)classloaders.get(codebase);
        if (c == null) {
            c = new AppletClassLoader(codebase);
            classloaders.put(codebase, c);
        }
        return c;
    }


    public Thread getAppletHandlerThread() {
        return handler;
    }

    public int getAppletWidth() {
        return appletSize.width;
    }

    public int getAppletHeight() {
        return appletSize.height;
    }

    // Debug code
    static void debug(String s) {
        if(debug)
            System.err.println("AppletPanel:::" + s);
    }

    static void debug(String s, Throwable t) {
        if(debug) {
            t.printStackTrace();
            debug(s);
        }
    }

    private static AppletMessageHandler amh = new AppletMessageHandler("appletpanel");

}
