/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//NOTTRANSLATABLE
package borland.jbcl.control.applet;

import java.net.URL;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.net.URLConnection;
import java.awt.*;
import java.applet.*;
import java.security.*;

import sun.awt.image.URLImageSource;
import sun.net.www.MessageHeader;
import sun.tools.jar.*;
import sun.misc.*;
//import sun.security.*;
//import sun.security.pkcs.*;
import sun.applet.AppletAudioClip;



/**
 * Integrate the loading of classes, images and audio for an applet.
 *
 * <p>The method loadJar() reads successive entries from a JarInputStream and
 * passes off the contents to the appropriate content handler, based
 * on the MIME-type of the entry.
 *
 * <p>In general, anything that implements AppletContext should
 * instantiate one of these for each Applet - each Applet should have
 * its own ResourceLoader.  The things that implement AppletContext now are
 * the Appletviewer and sun.hotjava.tags.TagAppletPanel.
 *
 * <p>This class will also eventually be aware of a persistent
 * client-side disk-cache for images, sounds and classes.
 *
 * @version 1.22 97/01/27
 * @author Benjamin Renaud
 * @author Dave Brown */

public class AppletResourceLoader {

    static boolean debug = false;
    static boolean verboseResourceLoading = false;

    private void verbose(String s) {
        if (verboseResourceLoading) {
            System.err.println(s);
        }
    }
    static void debug(String s) {
        if (debug)
            System.err.println(s);
    }

    static void debug(String s, Throwable t) {
        if (debug)
            t.printStackTrace();
            System.err.println("borland.jbcl.control.applet.AppletResourceLoader:::" + s);
    }

    /*
     * where we store images.  This will be the image hashtable
     * for either Hotjava or the appletviewer. We put AppletImageRef's
     * into this table.
     */

    static private Hashtable imgHash = new Hashtable();

    /*
     * Where we store AppletAudioClips.  This will be the audio
     * hashtable for the browser or appletviewer.  We'll put ONLY
     * objects of type AudioClip into this hashtable.
     */

    static private Hashtable audioHash = new Hashtable();

    /*
     * Resources as bytecode arrays
     */

    static private Hashtable resourceHash = new Hashtable();

    /*
     * Mime type per entry handled.
     */

    static private Hashtable mimeHash = new Hashtable();

    /*
     * The ClassLoader for this applet.  This is where we define
     * any classes we load from the Zip stream.
     */

    AppletClassLoader loader;

    /*
     * The identity scope where we look for signers.
     */
    IdentityScope scope;

    /*
     * constructor
     */

    public AppletResourceLoader (AppletClassLoader loader) {
        this.loader = loader;
        scope = IdentityScope.getSystemScope();
    }

    /**
     * Read entries from a Zip stream, decode them and
     * store in appropriate place (cache)
     * The <i>file</i> can be a full URL pointing
     * to a different host than the one the applet came
     * from - "http://java.sun.com/applets/baseClasses.jar"
     * for example
     * @param url the codebase for the applet
     * @param file the jar file
     */

    void loadJar (URL url, String file) throws IOException {

        debug("jarLoader: loading jar " + url + "/" + file);

        URL jarURL = new URL(url, file);
        JarVerifierStream jvs = null;

        /* mapping of names to objects that have been loaded from the jar */
        Hashtable nameToObject = new Hashtable();

        try {
            jvs = new JarVerifierStream(jarURL.openStream());
            ZipEntry ent = null;

            while ((ent = jvs.getNextEntry()) != null) {

                String name = ent.getName();
                String type = null;

                /* the object we're loading: the datum of nameToObject */
                Object loadedObj = null;
                ByteArrayOutputStream baos = new ByteArrayOutputStream();

                /* NOTE: We don't know the size of an entry until
                   we've reached the end of one because of
                   compression. This means we can't just do a get size
                   and read in the entry. */

                debug("loading: " + name + "...");

                BufferedInputStream bufis = new BufferedInputStream(jvs);
                int i = bufis.read();
                while(i != -1) {
                    baos.write(i);
                    i = bufis.read();
                }

                debug("done loading: " + name + ".");

                byte[] buf = baos.toByteArray();
                int size = buf.length;

                /* Parse entries which have special names, like the
                   manifest. We invent "mime types" local to this class */
                if (type == null) {
                    type = guessManifestType(name);
                }

                if (type == null) {
                    /*  REMIND Get this from MANIFEST.MF. This should
                        be changed as soon as manifest support is
                        fixed.  Because jvs must support mark, and it
                        isn't always obvious since it's a
                        FilterInputStream and whatever it used to
                        build it is what would give it mark. */
                    InputStream tmpStream = new ByteArrayInputStream(buf);
                    type = URLConnection.guessContentTypeFromStream(tmpStream);
                }

                if (type == null) {
                    // REMIND - this canbe cleaned up if we don't mind
                    // exporting access to MANIFEST data. - epll
                    type = "input-stream/input-stream";
                }

                debug("loading " + name + " (most likely guessed mime-type: " +
                      type +")");

                if (type.startsWith("image")) {
                    // REMIND - the image code should get it from the Image cache
                    putLocalResource(new URL(url, name), buf, type);
                    loadedObj = createImage(buf, url, name, type);
                } else if (type.startsWith("audio")) {
                    // REMIND - the image code should get it from the Audio cache
                    URL url2 = new URL(url, name);
                    putLocalResource(url2, buf, type);
                    AppletAudioClip aud = new AppletAudioClip(buf);
                    audioHash.put(url2, aud);
                    loadedObj = aud;

                } else if (type.startsWith("input-stream")) {
                    putLocalResource(new URL(url, name), buf, type);
                    ByteArrayInputStream bis = new ByteArrayInputStream(buf);
                    loadedObj = bis;

                } else if (type.startsWith("application/java-vm")) {
                    putLocalResource(new URL(url, name), buf, type);

                    /* remove the .class suffix */
                    String classname = name.substring(0, name.length() - 6).replace('/', '.');

                    /* define the class */
                    loadedObj = loader.defineClassFromBytes(classname, buf,
                                                            0, buf.length);

                } else if (type.equals("manifest/manifest") ||
                           type.equals("manifest/signature-bin")) {
                    /* We know what it is. It is handled by JarVerifierStream */

                } else {
                    debug("ZipEntry " + name +
                          " has unsupported mimetype " + type);
                }

                if (loadedObj != null) {

                    nameToObject.put(name, loadedObj);
                    loadedObj = null;
                }
            }

            /* If we ran in any signature blocks, we sign the relevant
               objects */
            Enumeration blocks = jvs.getBlocks();
            Hashtable nameToHash = jvs.getNameToHash();
            Manifest manifest = jvs.getManifest();
            if (blocks != null) {
                signObjects(jvs.getVerifiedSignatures(), nameToObject);
                //              signObjects(nameToObject, nameToHash, manifest, blocks);
            } else {
                debug("BLOCK is null");
            }

        } catch (IOException e) {
            debug("IOException loading archive: " + e);
            e.printStackTrace();

        } finally {
            debug("Closing jar input stream");
            if (jvs != null)
                jvs.close();
        }
    }

    private void signObjects(Hashtable signatures, Hashtable nameToObject) {

        Enumeration filenames = signatures.keys();

        while (filenames.hasMoreElements()) {
            String filename = (String)filenames.nextElement();
            Vector ids = (Vector)signatures.get(filename);
            Object obj = nameToObject.get(filename);

            Identity[] identities = new Identity[ids.size()];
            ids.copyInto(identities);

            /* Right now, we only sign classes. */
            if (obj instanceof AppletClassEntry) {

                AppletClassEntry e;
                if ((e = (AppletClassEntry) loader.rawClasses.get(obj))
                    != null &&  e == obj) {
                    e.ids = identities;
                }
            }
        }
    }


    String guessManifestType(String name) {
        name = name.toUpperCase();

        /* ignore leading slash */
        if (name.startsWith("/")) {
            name = name.substring(1);
        }
        /* a manifest file or signature file at all? */
        if (!name.startsWith("META-INF/")) {
            return null;
        }

        /* the manifest? */
        if (name.equalsIgnoreCase("META-INF/MANIFEST.MF")) {
            return "manifest/manifest";
        }

        /* A signature _block_ file? */
        if (name.endsWith("DSA") || name.endsWith("PK7") ||
            name.endsWith("PGP")) {
            return "manifest/signature-bin";
        }

        /* A regular signature file? */
        if (name.endsWith("SF")) {
            return "manifest/signature-asc";
        }
        return null;
    }

    /*
     * Handling of Audio resources
     */

    /**
     * Get a clip from the static audio cache.
     */
    static synchronized AudioClip getAudioClip(URL url) {
        System.getSecurityManager().checkConnect(url.getHost(), url.getPort());
        AudioClip clip = (AudioClip)audioHash.get(url);
        if (clip == null) {
            audioHash.put(url, clip = new AppletAudioClip(url));
        }
        return clip;
    }

    /**
     * Get Static Images
     */

    Image createImage(byte[] buf, URL url, String file, String type)
        throws java.net.MalformedURLException {
        debug("creating image: " + file);
        InputStream is =
            new BufferedInputStream(new ByteArrayInputStream(buf));
        JarImageSource src = new JarImageSource(url, is, type);

        Image img = Toolkit.getDefaultToolkit().createImage(src);
        URL key = new URL(url, file);
        AppletImageRef ref = new AppletImageRef(key);
        ref.setThing(img);
        imgHash.put(key, ref);

        return img;
    }

    public static synchronized Image getImage(URL url) {
        // May want to do a checkconnect here
        // System.getSecurityManager().checkConnect(url.getHost(), url.getPort());

        return (Image)getImageRef(url).get();
    }
    public static synchronized sun.misc.Ref getImageRef(URL url) {
        //pool.add(url.toExternalForm());

        AppletImageRef ref = (AppletImageRef)imgHash.get(url);
        if (ref == null) {
            ref = new AppletImageRef(url);
            imgHash.put(url, ref);
        }
        return ref;
    }
    public static void flushImages() {
        imgHash = new Hashtable();
    }


    /**
     * Set a resource. This should remain package private
     */
    void putLocalResource(URL url, byte[] data, String type) {
        resourceHash.put(url, data);
        mimeHash.put(url, type);
    }

    /**
     * Get an object from (caches/whatever) that this ResourceLoader is
     * managing.
     */

    public static synchronized Object getLocalResource(URL url) {
        Object o = mimeHash.get(url);
        if (o == null) {
            return  null;
        }
        String type = (String) o;
        if (type.startsWith("audio")) {
            // return an AudioClip
            if ((o = audioHash.get(url)) != null) {
                return o;
            }
        }
        // Image, something else, or audio for which we don't have a cache value
        byte[] buf = (byte []) resourceHash.get(url);

        // REMIND - on cache flushes, we may need to regenerate buf
        InputStream is =
            new BufferedInputStream(new ByteArrayInputStream(buf));

        if (type.startsWith("image")) {
            // return an ImageSource
            JarImageSource jis = new JarImageSource(url, is, type);
            return jis;
        }

        // something else, return an input stream
        return is;
    }

    public static InputStream getLocalResourceStream(URL url) {
        Object back = null;
        if ((back = resourceHash.get(url)) != null) {
            byte[] buf = (byte []) back;
            back = new BufferedInputStream(new ByteArrayInputStream(buf));
            return (InputStream) back;
        }
        // if we ever flush the resourceHash cache, then at this point
        // we would need to go pull the images back from the original
        // URL.
        return null;
    }

    /**
     * Return a URL to the desired resource.
     */
    public String getResourceAsName(String name) {
        verbose("AppletResourceLoader.getResourceAsName for name: "+name);
        URL base = loader.getCodeBase();
        URL url;
        try {
            url = new URL(base, name);
        } catch (Exception ex) {
            //  REMIND - should I give a different type of error?
            return null;
        }
        Object o = resourceHash.get(url);
        if (o != null) {
            verbose("Found resource: " + name +
                    " in some jar archive for base: "+base);
            // Create a URL to refer to this resource
            try {
                url = new URL("appletresource",
                              base.getHost(),
                              base.getPort(),
                              "/"+base.toExternalForm()+"/+/"+name);
                return url.toExternalForm();
            } catch (Exception e) {
                verbose("Exception "+e+" while building a resource URL");
                return null;
            }
        }
        // try to locate as an URL.
        return url.toExternalForm();
    }

    /**
     * Returns an InputStream on the resource
     */

    public InputStream getResourceAsStream(String name) {
        verbose("AppletResourceLoader.getResourceAsStream for name: "+name);
        URL base = loader.getCodeBase();
        URL url;
        try {
            url = new URL(base, name);
        } catch (Exception ex) {
            return null;
        }
        InputStream back = getLocalResourceStream(url);
        // check for all JAR files in archives
        if (back != null) {
            verbose("Found resource: " + name +
                    " in jar archive file");
            return back;
        }
        // try to locate as an URL.
        verbose("Opening stream to: " + url + " to get resource " + name);
        try {
            // most of the time it would be enough to do a
            // back = url.openStream();
            // but when the resource name ends with .html, .htm, or .txt,
            // HttpURLConnection will give a valid connection.
            // So, to cover all the cases:
            URLConnection conn = url.openConnection();
            if (conn instanceof java.net.HttpURLConnection) {
                java.net.HttpURLConnection hconn =
                    (java.net.HttpURLConnection) conn;
                int code = hconn.getResponseCode();
                if (code == java.net.HttpURLConnection.HTTP_OK) {
                    back = url.openStream();
                } if (code >= java.net.HttpURLConnection.HTTP_BAD_REQUEST) {
                    back = null;
                }
            } else {
                back = url.openStream();
            }

        } catch (Exception ex) {
            return null;
        }
        return back;
    }


    void resetLocalResources() {
        // REMIND -- there is really no support for this right now -epll
        resourceHash.clear();
    }

}

class HashValues {

    byte[] sha;
    byte[] md5;

    HashValues(byte[] sha, byte[] md5) {
        this.sha = sha;
        this.md5 = md5;
    }

}

