/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
package borland.jbcl.control.applet;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import borland.jbcl.layout.*;
import borland.jbcl.view.BorderItemPainter;

public class AppletViewerBevelPanel extends Panel 
{
  public static final int FLAT    = 0;
  public static final int RAISED  = 1;
  public static final int LOWERED = 2;

  public AppletViewerBevelPanel() {
    setLayout(new BorderLayout());
    setBackground(SystemColor.control);
  }

  public void setBevelInner(int bevelInner) {
    if ((bevelInner == FLAT || bevelInner == RAISED || bevelInner == LOWERED) && bevelInner != inner) {
      inner = bevelInner;
      resetEdges();
      validate();
    }
  }

  public void setBevelOuter(int bevelOuter) {
    if ((bevelOuter == FLAT || bevelOuter == RAISED || bevelOuter == LOWERED) && bevelOuter != outer) {
      outer = bevelOuter;
      resetEdges();
    }
  }
  
  public Insets getInsets() {
    Insets i = super.getInsets();
    Insets b = border != null ? border.getInsets() : new Insets(0,0,0,0);
    return new Insets(i.top + b.top + margins.top,
                      i.left + b.left + margins.left,
                      i.bottom + b.bottom + margins.bottom,
                      i.right + b.right + margins.right);
  }

  public void paint(Graphics g) {
    super.paint(g);
    if (g == null)
      return;

    if (border != null) {
      Rectangle rect = new Rectangle(0, 0, getSize().width, getSize().height);
      border.paint(null, g, rect, 0, null);
    }
  }

  protected void resetEdges() {
    if (inner == FLAT && outer == FLAT)
      border = null;
    else {
      int style = 0;
      switch (inner) {
        case FLAT:
          break;
        case RAISED:
          style |= BorderItemPainter.INNER_RAISED;
          break;
        case LOWERED:
          style |= BorderItemPainter.INNER_SUNKEN;
          break;
      }
      switch (outer) {
        case FLAT:
          break;
        case RAISED:
          style |= BorderItemPainter.OUTER_RAISED;
          break;
        case LOWERED:
          style |= BorderItemPainter.OUTER_SUNKEN;
          break;
      }
      border = new BorderItemPainter(style, BorderItemPainter.RECT, soft ? BorderItemPainter.SOFT : 0);
    }
    invalidate();
    repaint(100);
  }

  private int inner = FLAT;
  private int outer = RAISED;
  private boolean soft = false;
  private BorderItemPainter border = new BorderItemPainter(BorderItemPainter.RAISED);
  private Insets margins = new Insets(0,0,0,0);
}
