/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
// main wrapper
//
// Need a way to determine what this is dropped into.
//   Drop into applications only.
//   Display an error if dropped into or run in an applet.
//
//   from AppletSecurity.java:
//
//   "Applets are not allowed to create class loaders, or even execute any
//     of ClassLoader's methods." - AppletSecurityException("checkcreateclassloader")
//
//   "Applets are not allowed to manipulate thread groups outside
//     applet thread groups." -
//     AppletSecurityException("checkaccess.threadgroup", (ThreadGroup)g.toString())
//
//   "If not an AppletClassLoader, we don't know what to do." -
//     AppletSecurityException("checkread.unknown", file)

package borland.jbcl.control.applet;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import borland.jbcl.control.*;
import borland.jbcl.layout.*;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;

public class AppletViewerWrapperPanel extends AppletViewerBevelPanel implements AppletContext, borland.jbcl.util.BlackBox, Runnable
{
  public  String publicPackageName;

  private String className;
  private String urlPath;
  private String archiveName;
  private String codePath;
  private String finalurlPath;

  private Properties props;
  private boolean appletWasLoaded = false;
  private boolean appletChecksOut = false;
  private boolean localAppletFound = true;
  private boolean appletWasStarted = false;
  private boolean appletIsRunning = false;
  private boolean runInDesign = true;
  private boolean reloadApplet = false;

  private AppletViewerControlPanel panel;
  private TextField msglabel;
  private AppletViewerBevelPanel bevelPanel;
  private URL url;
  private URL urlTest;
  private Thread handler;

  public final void initPanel(String className, String archiveName, String urlPath, String codePath) {
    this.className = className;
    this.archiveName = archiveName;
    this.urlPath = urlPath;
    this.codePath = codePath;
  }

  public final void runWrappedAppletViewer(Properties properties) {
    setLayout(new BorderLayout());
    this.props = properties;
    setBevelInner(RAISED);
    setBevelOuter(LOWERED);
    msglabel = new TextField();
    add(msglabel, BorderLayout.CENTER);  //NORES
    msglabel.setBackground(new Color(225, 158, 121));
    msglabel.setBounds(0, 0, 200, 20);
    msglabel.setVisible(true);
    msglabel.setText(Res.format(Res.CheckingClass, new String[] {className}));

    if (archiveName != null)
      props.put("archive", archiveName);  //NORES
    if (codePath != null)
      props.put("codebase", codePath);  //NORES

    props.put("code", className);  //NORES
    if (urlPath != null) {
      finalurlPath = urlPath;
      localAppletFound = true;
      props.put("documentbase", urlPath);  //NORES
    }
    else {
      String s = getCanonicalPathFromClasspath();
      if (s == null)
        localAppletFound = false;
      else
        props.put("documentbase", s);  //NORES
    }
    runHandler();
  }

  final void runHandler() {
    if (localAppletFound) {
      try {
        url = new URL((String)props.get("documentbase"));  //NORES
        panel = new AppletViewerControlPanel(url, props);
        panel.init();
        add(panel, BorderLayout.CENTER);
      }
      catch (Exception e) {
        System.err.println("    **EXCEPTION: "+e);
      }
      catch (Error e) {
        System.err.println("    **ERROR: "+e);
      }

      // An applet's max priority is set to 4, but we want its default priority
      // to be set to 3.  This is so that a CPU-intensive thread that takes
      // the default priority won't end up starving system threads. For example,
      // ImageFetcher runs at P3, which is bad if a cpu-bound thread is
      // executing at P4, especially if the AWT thread is waiting for an image
      // to come in.  BUT 2 seems to work better when some applet's thread is
      // interrupted so that it doesn't affect other applets.
      panel.getAppletHandlerThread().setPriority(2);

      ThreadGroup appletGroup = panel.handler.getThreadGroup();
      handler = new Thread(appletGroup, this, "WrapperThread");  //NORES
      handler.setDaemon(true); //cheap insurance against a VCD failure if we die?

      // The default priority is 5 but the JBuilder designer may want something
      // higher when loading with many applets on it.
      handler.setPriority(7);
      panel.handler.start();
      handler.start();
    }
    else {
      System.err.println("    **Can't find: " + className);
      msglabel.setText(Res.format(Res.ClassNotFound, new String[] {className}));
    }
  }

  public void run() {
    synchronized(this) {
      checkURL();
      startApplet();
    }
  }

  final String getCanonicalPathFromClasspath() {
    URL url = null;

    File file = null;
    String path = System.getProperty("java.class.path");  //NORES
    StringTokenizer tokenizer = new StringTokenizer(path, File.pathSeparator);
    String packageName = className;
    String classNameWithoutPackage = null;

    packageName = packageName.substring(0, packageName.lastIndexOf("."));
    if (packageName.lastIndexOf(".") > -1) {
      packageName = packageName.substring(0, packageName.lastIndexOf("."));
      publicPackageName = packageName;
      classNameWithoutPackage = className.substring(packageName.length() + 1, className.length());
      packageName =  "\\" + packageName.replace('.', '\\');
    }
    else {
      packageName = null;
    }

    try {
      while (tokenizer.hasMoreTokens()) {
        path = tokenizer.nextToken();
        if (isZipFile(path)) {
          ZipFile zipFile = new ZipFile(path);
          if (publicPackageName != null) {
            String tempStr = publicPackageName.replace('.','/') + "/" + classNameWithoutPackage;
            ZipEntry zipEntry = zipFile.getEntry(tempStr);
            if (zipEntry != null) {
              String filePath = "file:/" + path.substring(0, path.lastIndexOf("\\")).replace('\\','/') + "/" + zipEntry.toString();
              url = new URL(filePath);
              finalurlPath = url.toString();
            }
          }
        }
        else {
          if (packageName == null) {
            file = new File(path + File.separator + className);
          }
          else {
            file = new File(path + packageName + File.separator + classNameWithoutPackage);
          }
          if (file.exists()) {
            try {
              int i = file.getCanonicalPath().lastIndexOf("\\", file.getCanonicalPath().length()-1);
              if (i >= 0) {
                String filePath = file.getCanonicalPath();
                if (packageName == null) {
                  filePath = "file:/" + filePath.replace('\\', '/');
                }
                else {
                  filePath = "file:/" + filePath.substring(0, filePath.length() - className.length()) + className;
                  filePath = filePath.replace('\\', '/');
                }
                url = new URL(filePath);
                finalurlPath = url.toString();
              }
            }
            catch(Exception e) {
              msglabel.setText("Error: "+e);
              System.err.println("    **EXCEPTION in getCanonicalPathFromClasspath: "+e);
              return null;
            }
          }
        }
      }

    }
    catch(Exception e) {
      msglabel.setText("Error: "+e);
      System.err.println("    **EXCEPTION in getCanonicalPathFromClasspath: "+e);
      return null;
    }
    if (url == null)
      return null;
    localAppletFound = true;
    return url.toString();
  }

  boolean isZipFile(String pathName) {
    int dot = pathName.lastIndexOf('.');
    if (dot != -1)
      return pathName.substring(dot+1).equalsIgnoreCase("zip");
    return false;
  }

  final void checkURL() {
    URL baseURL = null;
    URL documentURL;
    if (localAppletFound) {
      try {
        String att = className;
        documentURL = new URL(finalurlPath);
        if (att != null) {
          if (!att.endsWith("/")) {
            att += "/";
          }
          try {
            baseURL = new URL(documentURL, att);
          }
          catch (MalformedURLException e) {
          }
        }
        if (baseURL == null) {
          String file = documentURL.getFile();
          int i = file.lastIndexOf('/');
          if (i > 0 && i < file.length() - 1) {
            try {
              baseURL = new URL(documentURL, file.substring(0, i + 1));
            }
            catch (MalformedURLException e) {
            }
          }
        }
        if (!localAppletFound) {
          URLConnection urlCon = baseURL.openConnection();
          InputStream is = urlCon.getInputStream();
          is.close();
        }
        appletChecksOut = true;
      }
      catch (Exception e) {
        appletWasStarted = false;
        appletChecksOut = false;
        msglabel.setText("Error: "+className+", "+e);
        msglabel.setVisible(true);
        System.err.println("Could not start applet: "+e);
      }
    }
  }

  final void startApplet() {
    if (!appletWasStarted && appletChecksOut) {
      msglabel.setText("Loading: "+className);
      panel.sendEvent(panel.APPLET_LOAD);
      panel.appletSize = getSize();
      panel.sendEvent(panel.APPLET_INIT);
      if (!java.beans.Beans.isDesignTime()) {
        panel.sendEvent(panel.APPLET_START);
      }
      else {
        appletWasLoaded = true;
        if (runInDesign)
          panel.sendEvent(panel.APPLET_START);
        else
          msglabel.setText("APPLET: "+className);
      }
    }
  }

  public void changeAppletProperties() {
    try {
      if (appletWasStarted && appletChecksOut) {
        panel.sendEvent(AppletPanel.APPLET_STOP);
        setBevelInner(RAISED);
        setBevelOuter(LOWERED);
        msglabel.setText("Reloading: "+className);
        msglabel.setVisible(true);
        panel.sendEvent(AppletPanel.APPLET_DESTROY);
        panel.sendEvent(AppletPanel.APPLET_DISPOSE);
        panel.flushClassLoader(panel.baseURL);
        panel.getAppletHandlerThread().stop();
        panel.init();
        panel.handler.start();
        panel.sendEvent(panel.APPLET_LOAD);
        panel.appletSize = getSize();
        panel.sendEvent(panel.APPLET_INIT);
        if (!java.beans.Beans.isDesignTime()) {
          panel.sendEvent(panel.APPLET_START);
        }
        else {
          appletWasLoaded = true;
          if (runInDesign)
            panel.sendEvent(panel.APPLET_START);
          else
            msglabel.setText("APPLET: " + className);
        }
      }
    }
    catch (Exception e) {
      msglabel.setText("Exception: "+e);
      System.err.println("    **EXCEPTION in Reloading Applet: "+e);
    }
    catch (Error e) {
      msglabel.setText("Error: "+e);
      System.err.println("    **ERROR in Reloading Applet: "+e);
    }
  }

  public void restartLostApplet() {
    if (!appletWasStarted) {
      props.put("code", className);
      if (urlPath != null) {
        finalurlPath = urlPath;
        props.put("documentbase", urlPath);
        localAppletFound = true;
      }
      else {
        props.put("documentbase", getCanonicalPathFromClasspath());
      }
      if (archiveName != null)
        props.put("archive", archiveName);
      if (codePath != null)
        props.put("codebase", codePath);
    }
    if (java.beans.Beans.isDesignTime())
      runHandler();
  }

  public String get_ClassName() {
    return className;
  }

  public void set_ClassName(String s) {
    className = s;
    restartLostApplet();
  }

  public String get_urlPath() {
    return urlPath;
  }

  public void set_urlPath(String s) {
    urlPath = s;
    restartLostApplet();
  }

  public String get_CodeBase() {
    return codePath;
  }

  public void set_CodeBase(String s) {
    codePath = s;
    restartLostApplet();
  }

  public boolean get_ReloadApplet() {
    return reloadApplet;
  }

  public void set_ReloadApplet(boolean b) {
    reloadApplet = b;
    reloadApplet = false;
    changeAppletProperties();
  }

  public final boolean get_DesignRun() {
    return runInDesign;
  }

  public final void set_DesignRun(boolean b) {
    runInDesign = b;
    if (appletWasLoaded) {
      if (b) {
        panel.sendEvent(panel.APPLET_START);
        msglabel.setVisible(false);
        setBevelInner(FLAT);
        setBevelOuter(FLAT);
        appletIsRunning = true;
        runInDesign = true;
      }
      else {
        panel.sendEvent(panel.APPLET_STOP);
        msglabel.setText("Applet is stopped.");
        setBevelInner(RAISED);
        setBevelOuter(LOWERED);
        msglabel.setVisible(true);
        appletIsRunning = false;
        runInDesign = false;
      }
    }
  }

  public void showDocument(URL url) {
  }

  public void showDocument(URL url, String target) {
  }

  public Applet getApplet(String name) {
    return null;
  }

  public Enumeration getApplets() {
    return null;
  }

  public void showStatus(String status) {
    msglabel.setText(status);
    if (status == "Applet loaded.") {
      msglabel.setText("Applet still loading.");
    }
    if (status == "OK") {
      msglabel.setVisible(false);
      setBevelInner(FLAT);
      setBevelOuter(FLAT);
      appletWasStarted = true;
      appletIsRunning = true;
      panel.getApplet().setSize(getSize());
    }
    if (status == "NOOP") {
      msglabel.setText(className+" failed to init");
    }
    if (!runInDesign && java.beans.Beans.isDesignTime()) {
      setBevelInner(RAISED);
      setBevelOuter(LOWERED);
      msglabel.setVisible(true);
    }
  }

  private Hashtable audioHash = new Hashtable();

  public AudioClip getAudioClip(URL url) {
    AudioClip clip = (AudioClip)audioHash.get(url);
    if (clip == null) {
      clip = new sun.applet.AppletAudioClip(url);
      audioHash.put(url, clip);
    }
    return clip;
  }

  public Image getImage(URL url) {
    return getToolkit().getImage(url);
  }

  public boolean handleEvent(Event evt) {
    switch (evt.id) {
      case panel.APPLET_RESIZE:
        validate();
        return true;
      default: break;
    }
    return super.handleEvent(evt);
  }

  // needed to unload applet when deleted from VCD (audio clips keep running)
  protected void finalize() throws Throwable {
    panel.sendEvent(AppletPanel.APPLET_STOP);
    panel.sendEvent(AppletPanel.APPLET_DESTROY);
    panel.sendEvent(AppletPanel.APPLET_DISPOSE);
    panel.sendEvent(AppletPanel.APPLET_QUIT);
    panel.getAppletHandlerThread().stop();
    super.finalize();
  }

}
