/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.dataset;

import borland.jbcl.util.ErrorResponse;
import borland.jbcl.util.Variant;
import java.util.*;

/**
    Row editing before and after events for DataSets.
*/
public interface EditListener extends EventListener
{
  /** Called when the editing of a new or existing row in a DataSet is about to be
      canceled.  An application might use this event to save off undo information.
  */
  void canceling(DataSet dataSet) throws Exception;

  /** updating is called before a modified row is posted to the DataSet.  If an Exception
      is thrown inside this method, the post operation will not be performed.
      ValidationException will be thrown instead.

      updating is called before checks to make sure all required fields are non null.

      If a VetoException is constructed with a String parameter, this String will
      be used in the default error handling displays.
      (ie throw new VetoException("My error message"))
  */
  void updating(DataSet dataSet, ReadWriteRow newRow, ReadRow oldRow) throws Exception;
  /** Called after a row has been successfully posted to a DataSet.
  */
  void updated(DataSet dataSet) throws DataSetException;


  /** adding is called before a row is added to the dataSet.  If an Exception
      is thrown inside this method, the post operation will not be performed.
      ValidationException will be thrown instead.

      adding is called before checks to make sure all required fields are non null.

      If a VetoException is constructed with a String parameter, this String will
      be used in the default error handling displays.
      (ie throw new VetoException("My error message"))
  */
  void adding(DataSet dataSet, ReadWriteRow newRow) throws Exception;
  /** added is called after a row is successfully added to the dataSet.
  */
  void added(DataSet dataSet) throws DataSetException;


  /** deleting is called before a row is added to the dataSet.  If an Exception
      is thrown inside this method, the delete operation will not be performed.
      ValidationException will be thrown instead.


      If a VetoException is constructed with a String parameter, this String will
      be used in the default error handling displays.
      (ie throw new VetoException("My error message"))
  */
  void deleting(DataSet dataSet) throws Exception;
  /** Called after a successful delete operation has been performed.
  */
  void deleted(DataSet dataSet) throws DataSetException;

  /** Called just before a DataSet row is about to go into edit mode.
  */
  void modifying(DataSet dataSet) throws Exception;

  /** Called just before a dataSet attempts to insert a new unposted row.
  */
  void inserting(DataSet dataSet) throws Exception;
  /** Called just after a dataSet inserts a new unposted row.  This event
      can be used to initialize row values of new rows.
  */
  void inserted(DataSet dataSet) throws DataSetException;

  /** Called when any exceptions occur setting a column value.  This includes validation
      check failures as well as any Exceptions thrown by a ColumnChangeListener.validating()
      event handler.  The ErrorResponse object allows the user to indicate how the error should be
      handled.

      Calling response.abort() causes the operation to fail with an appropriate
      DataSetException or ValidationException. Calling response.retry() causes the operation
      to be retried.  Calling response.ignore causes the operation to silently fail without
      an exception being thrown.
  */
  void editError(DataSet dataSet, Column column, Variant value, DataSetException ex, ErrorResponse response) throws DataSetException;

  /** Exceptions thrown for row adds can be filtered here.

      Calling response.abort() causes the operation to fail with an appropriate
      DataSetException or ValidationException. Calling response.retry() causes the operation
      to be retried.  Calling response.ignore causes the operation to silently fail without
      an exception being thrown.
  */
  void addError(DataSet dataSet, ReadWriteRow row, DataSetException ex, ErrorResponse response) throws DataSetException;

  /** Exceptions thrown for row changes can be filtered here.

      Calling response.abort() causes the operation to fail with an appropriate
      DataSetException or ValidationException. Calling response.retry() causes the operation
      to be retried.  Calling response.ignore causes the operation to silently fail without
      an exception being thrown.
  */
  void updateError(DataSet dataSet, ReadWriteRow row, DataSetException ex, ErrorResponse response) throws DataSetException;

  /** Exceptions thrown for row deletes can be filtered here.

      Calling response.abort() causes the operation to fail with an appropriate
      DataSetException or ValidationException. Calling response.retry() causes the operation
      to be retried.  Calling response.ignore causes the operation to silently fail without
      an exception being thrown.
  */
  void deleteError(DataSet dataSet, DataSetException ex, ErrorResponse response) throws DataSetException;

}
