/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.dataset;

import java.util.EventListener;
import borland.jbcl.util.ErrorResponse;

public interface ResolverListener extends EventListener
{
  public void insertingRow(ReadWriteRow row, ResolverResponse response) throws DataSetException;
  public void deletingRow(ReadWriteRow row, ResolverResponse response) throws  DataSetException;
  public void updatingRow(ReadWriteRow row, ReadRow oldRow, ResolverResponse response)  throws DataSetException;

  public void insertedRow(ReadWriteRow row) throws DataSetException;
  public void deletedRow(ReadWriteRow row) throws DataSetException;
  public void updatedRow(ReadWriteRow row, ReadRow oldRow) throws DataSetException;

  /**
    @param dataSet  The original DataSet passsed in to be resolved.  This can be used
                    to position the any controls bound to it if user interaction is needed.

    @param row      The row with the problem.  This can be modified to correct the problem
                    and retry the operation.

    @param ex       The exception that caused the error.  Note that this may be a chained
                    exception.  @see DataSetException.getExceptionChain().

    @param response Specify abort/ignore/retry response for this error.
                    @see borland.jbcl.util.ErrorResponse.  Note that an abort response causes
                    the all insert/update/delete operations in the same transaction to be
                    rolled back.

  */
  public void insertError(DataSet dataSet, ReadWriteRow row, DataSetException ex, ErrorResponse response) throws DataSetException;

  /**
    @param dataSet  The original DataSet passsed in to be resolved.  This can be used
                    to position the any controls bound to it if user interaction is needed.

    @param row      The row with the problem.  This can be modified to correct the problem
                    and retry the operation.

    @param ex       The exception that caused the error.  Note that this may be a chained
                    exception.  @see DataSetException.getExceptionChain().

    @param response Specify abort/ignore/retry response for this error.
                    @see borland.jbcl.util.ErrorResponse.  Note that an abort response causes
                    the all insert/update/delete operations in the same transaction to be
                    rolled back.

  */
  public void deleteError(DataSet dataSet, ReadWriteRow row, DataSetException ex, ErrorResponse response) throws  DataSetException;

  /**
    @param dataSet  The original DataSet passsed in to be resolved.  This can be used
                    to position the any controls bound to it if user interaction is needed.

    @param row      The row with the problem.  This can be modified to correct the problem
                    and retry the operation.

    @param oldRow   The original row.

    @param updRow   The row to use for the next update query.  At input this is a copy of
                    the original row.  Pass this to DataSet.refetchRow().

    @param ex       The exception that caused the error.  Note that this may be a chained
                    exception.  @see DataSetException.getExceptionChain().

    @param response Specify abort/ignore/retry response for this error.
                    @see borland.jbcl.util.ErrorResponse.  Note that an abort response causes
                    the all insert/update/delete operations in the same transaction to be
                    rolled back.

  */
  public void updateError(DataSet dataSet, ReadWriteRow row, ReadRow oldRow, ReadWriteRow updRow, DataSetException ex, ErrorResponse response)  throws DataSetException;
}

