/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
///--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import java.io.*;
import borland.jbcl.util.Diagnostic;

public class AsciiInputStream extends SimpleCharInputStream {

  private InputStream in;
  private byte[]      buffer;
  private int         count;
  private int         backCount;
  private int         ch;
  private int         pos;
  private int         undoChar;
  private int         undoPos;

  /**
   * Creates a new buffered stream with a default
   * buffer size.
   * @param in        the input stream
   */
  public AsciiInputStream(InputStream in) {
    this(in, 2048);
  }

  /**
   * Creates a new buffered stream with the specified buffer size.
   * @param in        the input stream
   * @param size      the buffer size
   */
  public AsciiInputStream(InputStream in, int size) {
    this.in = in;
    pos     = -1;
    undoPos = -1;
    buffer  = new byte[size];
  }

  private final void fill() throws IOException {
    pos = -1;
    count = in.read(buffer, 0, buffer.length);
    if (count < 0)
      count = pos;
  }

  /**
   * Reads a byte of data. This method will block if no input is available.
   * @return  the byte read, or -1 if the end of the stream is reached.
   * @exception IOException If an I/O error has occurred.
   * See encoding explanation in AsciiOutputStream.write().
   */
  public int read() throws IOException {
    if (++pos < count) {
      ch = buffer[pos] & 0xff;
      if (ch == '\\' ) {
        ch  = readByte();
        if (ch == '\\')
          return ch;
        else if (ch == 'u') {
          int result  = 0;
          loop: for (int i = 0 ; i < 4 ; i++) {
            ch = readByte();
            switch (ch) {
              case '0': case '1': case '2': case '3': case '4':
              case '5': case '6': case '7': case '8': case '9':
                result = (result << 4) + ch - '0';
                break;
              case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
                result = (result << 4) + 10 + ch - 'a';
                break;
              case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
                result = (result << 4) + 10 + ch - 'A';
                break;
              default:
                break loop;
            }
          }
          return result;
        }
        else {
          --pos;        // undoes last readByte().
          return '\\';
        }
      }
      else
        return ch;
    }
    else {
      if (undoPos != -1) {
        pos     = undoPos;
        undoPos = -1;
        return undoChar;
      }
      fill();
      if (pos < count)
        return read();
      return -1;
    }
  }

  private int readByte() throws IOException {
    if (++pos < count)
      return buffer[pos] & 0xff;
    fill();
    if (pos < count)
      return readByte();
    return -1;
  }

  public void unread(int undoChar) throws IOException {
    this.undoChar = undoChar;
    this.undoPos  = pos;
    pos           = count;
  }

  public void close() throws IOException {
    in.close();
  }
}
