/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
///--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import borland.jbcl.util.Hex;

/**
 *  This class provides static functions that help in reading
 *  Java encoding (Unicode escapes) in text files.
 */
public final class ByteToCharJava {

 /**
  * Returns a byte array containing the Unicode escape
  * of the given character.
  */
  public static final byte[] toUnicodeEscape(char chIn) {
    int in = (int) chIn;
    byte [] out = new byte[6];
    out[0] = '\\';
    out[1] = 'u';
    out[2] = Hex.bytes[in>>12];
    out[3] = Hex.bytes[(in>>8) & 0xf];
    out[4] = Hex.bytes[(in>>4) & 0xf];
    out[5] = Hex.bytes[in & 0xf];
    return out;
  }

  private static final int hexdigit2int(char ch) {
    if ('0' <= ch && ch <= '9')
      return ch - '0';
    if ('A' <= ch && ch <= 'F')
      return ch - 'A' + 10;
    if ('a' <= ch && ch <= 'f')
      return ch - 'a' + 10;
    return -1;
  }

 /**
  * Parses a string, converting Unicode escapes to the corresponding Unicode 
  * characters.  
  * 
  * Returns the resulting Unicode encoded string.
  * @param srcString the string to be converted.
  */
  public static final String parseUnicodeEscapes(String srcString) {
    char[] source = srcString.toCharArray();
    int resultLen = parseUnicodeEscapes(source, 0, source.length);
    if (resultLen != srcString.length()) {
      return new String(source, 0, resultLen);
    }
    else
      return srcString;
  }

 /**
  * Parses a char array, converting Unicode escapes to the 
  * corresponding Unicode characters.  
  * 
  * Returns the resulting number of 
  * unicode characters from the conversion.
  * @param source  the input char array
  * @param start   the starting position to convert
  * @param end     the offset to stop converting
  */
  public static final int parseUnicodeEscapes(char[] source, int start, int end) {
    char[] dest = source;
    int i = start;
    int j = start;
    int bslashes = 0;
    while (i < end) {
      if (source[i] == '\\') {
        bslashes++; // Unicode escapes are only parsed for an *odd*
                    // number of backslashes.
        if (bslashes%2!=0 && i + 1 < end && source[i+1] == 'u') {
          bslashes = 0;
          int l = i + 2; // skip over the '\' and the first 'u'
          while (l < end && source[l] == 'u') // and any subsequent u's
            l++;                              
          if (l + 4 <= end) {                 // process exactly 4 hex digits, ignore if too near end
            int code = 0;
            int d = 0;
            for (int k = 0; k < 4 && d >= 0; k++) {
              d = hexdigit2int(source[l + k]);
              code = code * 16 + d;
            }
            if (d >= 0) {                     // got exactly 4 good hex digits
              dest[j++] = (char)code;
              i = l + 4;
              continue;
            }
          }
        }
      }
      else {
        bslashes = 0;
      }
      dest[j++] = source[i++];
    }
    return j - start;
  }

 /**
  * Parses a string, converting escapes other than
  * Unicode escapes, to the correspoding control
  * characters. \n \t \b \r \f are converted.
  * Returns the resulting Unicode encoded string.
  * @param srcString the string to be converted.
  */
  public static final String parseEscapes(String srcString) {
    int len = srcString.length();
    char[] source = srcString.toCharArray();
    char[] dest = source;
    int j = 0;
    int i = 0;
    while (i < len) {
      if (source[i] == '\\' && i + 1 < len) {
        i++;
        switch (source[i]) {
          case 'n':
            dest[j++] = '\n'; 
            i++; 
            continue;
          case 't':
            dest[j++] = '\t'; 
            i++; 
            continue;
          case 'b':
            dest[j++] = '\b'; 
            i++; 
            continue;
          case 'r':
            dest[j++] = '\r'; 
            i++; 
            continue;
          case 'f':
            dest[j++] = '\f'; 
            i++; 
            continue;
          default:
        }
      }
      dest[j++] = source[i++];
    }
    if (i != j)
      return new String(dest, 0, j);
    return srcString;
  }
}
