/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import java.io.*;
import sun.io.*;
import borland.jbcl.util.Diagnostic;

public class EncodedInputStream extends SimpleCharInputStream {

  private InputStream in;
  private byte[]      bufferB;
  private char[]      bufferC;
  private int         count;
  private int         ch;
  private int         pos;
  private int         undoChar;
  private int         undoPos;
  private ByteToCharConverter byteToChar;
  private byte        savedFirstByte; //temp: part of a JDK bug workaround
                                      //temp: see fill()
  private static final int DEFAULTBUFFERSIZE = 2048;

  /**
   * Creates a new buffered stream with the default encoding and
   * with a default buffer size.
   * @param in        the input stream
   */
  public EncodedInputStream(InputStream in) throws UnsupportedEncodingException {
//    this(in, System.getProperty("file.encoding"));  //NORES
      this(in, (new OutputStreamWriter(System.out)).getEncoding()); // "file.encoding" is not a published property in
                                                                    // some browsers.
  }

  /**
   * Creates a new buffered stream with the specified encoding and
   * with a default buffer size.
   * @param in        the input stream
   */
  public EncodedInputStream(InputStream in, String encoding) throws UnsupportedEncodingException {
    this(in, encoding, DEFAULTBUFFERSIZE);
  }

  /**
   * Creates a new buffered stream with the specified
   * buffer size.
   * @param in         the input stream
   * @param size      the buffer size
   */
  public EncodedInputStream(InputStream in, String encoding, int size) throws UnsupportedEncodingException {
    this.in           = in;
    byteToChar        = ByteToCharConverter.getConverter(encoding);
    pos               = -1;
    undoPos           = -1;
    bufferB           = new byte[size];
    bufferC           = new char[size*byteToChar.getMaxCharsPerByte()];
  }

  private final void fill() throws IOException {
    // This code includes a workaround for a JDK bug:
    // When a multibyte character straddles a buffer (i.e.
    // trailbyte is first byte of a buffer), an ArrayIndexOutOfBounds
    // occurs in ByteToCharSJIS.convert().
    pos = -1;
    int bytes = in.read(bufferB, 0, bufferB.length-1);  //temp: -1

    if (bytes <= 0) {
      count = pos;
      return;
    }

    try {                                                         //temp
      count = byteToChar.convert(bufferB, 0, bytes, bufferC, 0, bufferC.length);
      savedFirstByte = bufferB[bytes-1];                            //temp
    }                                                             //temp
    catch (ArrayIndexOutOfBoundsException e) {                    //temp
      // this is a workaround for a JDK bug                       //temp
      byteToChar.reset();                                         //temp
      System.arraycopy(bufferB, 0, bufferB, 1, bytes++);          //temp
      bufferB[0] = savedFirstByte;                                //temp
      count = byteToChar.convert(bufferB, 0, bytes, bufferC, 0, bufferC.length);
    }                                                             //temp
    if (count <= 0)
      count = pos;
  }

  /**
   * Reads a byte of data. This method will block if no input is
   * available.
   * @return  the byte read, or -1 if the end of the
   *          stream is reached.
   * @exception IOException If an I/O error has occurred.
   * See encoding explanation in EncodedOutputStream.write().
   */

  private int readX() throws IOException {
    if (++pos < count) {
      ch = bufferC[pos] & 0xffff;
      return ch;
    }
    else {
      if (undoPos != -1) {
        pos     = undoPos;
        undoPos = -1;
        return undoChar;
      }
      fill();
      if (pos < count)
        return read();
      return -1;
    }
  }

  public int read() throws IOException {
    if (++pos < count) {
      ch = bufferC[pos];
      if (ch == '\\' ) {
        ch  = readX();
        if (ch == '\\')
          return ch;
        else if (ch == 'u') {
          int result  = 0;
          loop: for (int i = 0 ; i < 4 ; i++) {
            ch = readX();
            switch (ch) {
              case '0': case '1': case '2': case '3': case '4':
              case '5': case '6': case '7': case '8': case '9':
                result = (result << 4) + ch - '0';
                break;
              case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
                result = (result << 4) + 10 + ch - 'a';
                break;
              case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
                result = (result << 4) + 10 + ch - 'A';
                break;
              default:
                break loop;
            }
          }
          return result;
        }
        else {
          --pos;        // undoes last readX().
          return '\\';
        }
      }
      else
        return ch;
    }
    else {
      if (undoPos != -1) {
        pos     = undoPos;
        undoPos = -1;
        return undoChar;
      }
      fill();
      if (pos < count)
        return read();
      return -1;
    }
  }


  public void unread(int undoChar) throws IOException {
    this.undoChar = undoChar;
    this.undoPos  = pos;
    pos           = count;
  }

  public void close() throws IOException {
    in.close();
  }
}
