/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import java.io.*;
import sun.io.*;
import borland.jbcl.util.Diagnostic;
import borland.jbcl.util.Hex;

public class EncodedOutputStream extends SimpleCharOutputStream {

  private char[]        bufferIn;
  private byte[]        bufferOut;
  private CharToByteConverter charToByte;
  private OutputStream  out;
  private int           charSize;
  private int           backCount;

  /**
   * Creates output stream encoded in the default encoding.
   * @param out               the output stream
   * @param encodingString    the encoding in the resulting stream
   */
  public EncodedOutputStream(OutputStream out) throws UnsupportedEncodingException
  {
//    this(out, System.getProperty("file.encoding"));  //NORES
      this(out, (new OutputStreamWriter(System.out)).getEncoding());  // "file.encoding" is not a published property in
  }                                                                   // some browsers.

  /**
   * Creates output stream encoded in the specified encoding.
   * Unicode characters passed in by for a write() will be converted
   * to native, potentially multibyte, forms.
   * Characters that cannot be encoded in the specified encoding
   * will be represented in Unicode escapes.
   * @param out               the output stream
   * @param encodingString    the encoding in the resulting stream
   */
  public EncodedOutputStream(OutputStream out, String encodingString) throws UnsupportedEncodingException
  {
    this.out          = out;
    charToByte        = CharToByteConverter.getConverter(encodingString);
    charToByte.setSubstitutionMode(false);
    this.charSize     = charToByte.getMaxBytesPerChar();
    bufferOut         = new byte[charSize];
    bufferIn          = new char[1];
  }

  /**
   * Creates output stream encoded in the specified encoding
   * and the specified buffer size.  Currently this class
   * does not do its own buffering -- the buffer size is ignored.
   * Unicode characters passed to write() will be converted
   * to native, potentially multibyte, forms.
   * @param out               the output stream
   * @param encodingString    the encoding in the resulting stream
   */
  public EncodedOutputStream(OutputStream out, String encodingString, int bufferSize)
      throws UnsupportedEncodingException
  {
    this(out, encodingString);
  }

  /**
   * Writes a char. This method will block until the byte is actually
   * written.
   * @param b the byte to be written
   * @exception IOException If an I/O error has occurred.
   * Destination is encoded with the specified character set.
   * Characters that cannot be encoded in the specified character
   * set will be encode with Unicode escapes, like \u306e.
   * Malformed (invalid in *any* encoding) Unicode characters will
   * be converted to '?'.
   * Escaping (with \) rules are the same as with AsciiOutputStream.
   */
  public void write(int ch) throws IOException
  {
    bufferIn[0] = (char) ch;

    if (ch == '\\') {
      out.write('\\');
      if (++backCount > 1) {
        out.write('\\');
        out.write('\\');
        backCount = 0;
      }
    }
    else if ((backCount>0) && (ch=='u')) {
					out.write('\\');
					out.write('u');
					backCount = 0;
		}
		else {
      try {
        int size = charToByte.convert(bufferIn, 0, 1, bufferOut, 0, charSize);
        out.write(bufferOut, 0, size);
				backCount = 0;
      }
      catch (UnknownCharacterException e) {
        if (backCount > 0) {
          out.write('\\');
          backCount = 0;
        }
        out.write('\\');
        out.write('u');
        out.write(Hex.chars[(ch >> 12) & 0xF]);
        out.write(Hex.chars[(ch >>  8) & 0xF]);
        out.write(Hex.chars[(ch >>  4) & 0xF]);
        out.write(Hex.chars[ch & 0xF]);
      }
      catch (MalformedInputException e) {
        // For unconvertible characters, write a '?'
        // These are illegal Unicode characters, not characters
        // that are outside the target charset range.
				backCount = 0;
        out.write('?');
      }
    }
  }

  public void flush() throws IOException {
    out.flush();
  }

  public void close() throws IOException {
    out.flush();
    out.close();
  }
}
