/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
///--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;
import borland.jbcl.util.Diagnostic;
import sun.io.*;
import java.io.*;

public class FastBufferedOutputStream extends FilterOutputStream {
  /**
   * The buffer where data is stored.
   */
  protected byte[] buf;
  
  /**
   * The number of bytes in the buffer.
   */
  protected int count;
  
  /**
   * Creates a new buffered stream with a default
   * buffer size.
   * @param out       the output stream
   */
  public FastBufferedOutputStream(OutputStream out) {
    this(out, 2048);
  }
  
  /**
   * Creates a new buffered stream with the specified
   * buffer size.
   * @param out               the output stream
   * @param size      the buffer size
   */
  public FastBufferedOutputStream(OutputStream out, int size) {
    super(out);
    buf = new byte[size];
  }
  
  //
  // **** UNSYNCHRONIZED FOR SPEED ****
  //
  /**
   * Writes a byte. This method will block until the byte is actually
   * written.
   * @param b the byte to be written
   * @exception IOException If an I/O error has occurred.
   */
  public void write(int b) throws IOException {
    if (count == buf.length) {
      flush();
    }
    buf[count++] = (byte)b;
  }

  //
  // **** UNSYNCHRONIZED FOR SPEED ****
  //
  /**
   * Writes a subarray of bytes.
   * @param b the data to be written
   * @param off       the start offset in the data
   * @param len       the number of bytes that are written
   * @exception IOException If an I/O error has occurred.
   */
  public void write(byte b[], int off, int len) throws IOException {

    // More optimal than BufferedOutputStrea which tends to do more writes, and
    // more unaligned writes.
    //
    int avail = buf.length - count;

    while(len > avail) {
      write(b, off, avail);
      flush();
      off   += avail;
      len   -= avail;
      avail =  buf.length - count;
    }

    if (len > 0) {
      System.arraycopy(b, off, buf, count, len);
      count += len;
    }

  }

  //
  // **** UNSYNCHRONIZED FOR SPEED ****
  //
  /**
   * Flushes the stream. This will write any buffered
   * output bytes.
   * @exception IOException If an I/O error has occurred.
   */
  public void flush() throws IOException {
    if (count > 0) {
      out.write(buf, 0, count);
      out.flush();
      count = 0;
    }
  }
}
