/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
///--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import java.io.*;

/**
 * This is an abstract class that provides subclasses with
 * the capability of reading input one line at a time.
 * This class is used by AsciiEncodedInputStream and 
 * AsciiEncodedOutputStream.
 */
abstract public class SimpleCharInputStream {

  private char[] lineBuffer       = new char[0];
  private int    lineBufferLength = 0;

  abstract public int   read() throws IOException;
  abstract public void  unread(int ch) throws IOException;
  abstract public void  close() throws IOException;

  /**
   * Reads a complete line of text.  A complete line
   * is defined as a line terminated with a line-feed
   * character, or a line terminated with carriage-
   * return/line-feed pair.  Carriage returns
   * that are not immediately followed by a line
   * feed character are not treated as line terminators,
   * but as part of the current line.
   */
  public String readLine() throws IOException {
    int offset = 0;
    while (true) {
      int ch = read();
      switch (ch) {
        case -1:
          if (offset == 0)
            return null;
          return new String(lineBuffer, 0, offset);
        case '\r':
          int ch2 = read();
          if (ch2 != '\n')
            unread(ch2);
        case '\n':
          return new String(lineBuffer, 0, offset);
        default:
          if (offset >= lineBufferLength) {
            char[] newBuffer = new char[lineBufferLength+128];
            System.arraycopy(lineBuffer, 0, newBuffer, 0, offset);
            lineBuffer       = newBuffer;
            lineBufferLength = lineBuffer.length;
          }
          lineBuffer[offset++] = (char)ch;
          break;
      }
    }
  }
}
