/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
///--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.io;

import java.io.*;

/**
 * This is an abstract class that provides subclasses
 * with the ability to write complete strings, 
 * arbitrarily-delimited strings and terminated
 * lines.
 */
abstract public class SimpleCharOutputStream {

  abstract public void write(int ch) throws IOException;
  abstract public void close() throws IOException;

  /**
   * Writes out all the characters in the given string
   */
  public void write(String string) throws IOException {
    int length = string.length();
    for (int i = 0; i < length; i++)
      write(string.charAt(i));
  }

  /**
   * Writes a string with the given delimiter.  Each
   * occurrence of the given delimiter is escaped by
   * repeating the delimiter.
   */
  public void writeDelimited(String string, char delimiter) throws IOException {
    int length = string.length();
    write(delimiter);
    for (int i = 0; i < length; i++) {
      int ch = string.charAt(i);
      if (ch == delimiter)
        write(delimiter);
      write(ch);
    }
    write(delimiter);
  }

  /**
   * Writes out the given string, and follow
   * it with a line feed delimiter.  The line delimiter
   * in this case is a fixed line-feed character, regardless
   * of the line-terminating convention of the
   * current environment.
   */
  public void writeln(String string) throws IOException {
    write(string);
    writeln();
  }

  /**
   * Writes out a line delimeter.  The line delimiter
   * in this case is a fixed line-feed character, regardless
   * of the line-terminating convention of the
   * current environment.
   */
  public void writeln() throws IOException {
    write('\n');
  }
}
