/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.model;

import java.io.*;
import com.objectspace.jgl.*;
import borland.jbcl.util.*;

/**
 * The BasicVectorContainer is a basic implementation of WritableVectorModel
 * that uses jgl.Array to store data.
 */
public class BasicVectorContainer implements WritableVectorModel, Serializable
{
  private static final long serialVersionUID = 200L;

  public BasicVectorContainer()                        { array = new Array(); }
  public BasicVectorContainer(int size)                { array = new Array(size); }
  public BasicVectorContainer(int size, Object object) { array = new Array(size, object); }
  public BasicVectorContainer(Object[] newArray)       { array = new Array(newArray); }
  public BasicVectorContainer(Array newArray)          { array = new Array(newArray); }

  public Object[] getItems() {
    Object[] items = new Object[array.size()];
    array.copyTo(items);
    return items;
  }

  public void setItems(Object[] newItems) {
    array = new Array(newItems);
    processModelEvent(new VectorModelEvent(this, VectorModelEvent.STRUCTURE_CHANGED));
  }

  // VectorModel implementation

  public Object get(int index) {
    if (index >= 0 && array.size() > index)
      return array.at(index);
    return null;
  }

  public int getCount() {
    return array.size();
  }

  public int find(Object data) {
    return array.indexOf(data);
  }

  public void setCount(int count) {
    int size = array.size();
    if (!variableSize || count == size)
      return;
    else if (count > size)
      for (int i = size; i < count; i++)
        array.add(null);
    else
      for (int i = size; i > count; i--)
        array.remove(i-1);
  }

  public void addModelListener(VectorModelListener listener) {
    modelListeners.add(listener);
  }

  public void removeModelListener(VectorModelListener listener) {
    modelListeners.remove(listener);
  }

  // WritableVectorModel implementation

  public boolean canSet(int index, boolean startEdit) {
    return index < array.size();
  }

  public void set(int index, Object object) {
    array.put(index, object);
    processModelEvent(new VectorModelEvent(this, VectorModelEvent.ITEM_CHANGED, index));
  }

  public void touched(int index) {
    if (index < array.size())
      processModelEvent(new VectorModelEvent(this, VectorModelEvent.ITEM_TOUCHED, index));
  }

  public boolean isVariableSize() { return variableSize; }
  public void setVariableSize(boolean variable) { variableSize = variable; }

  public void addItem(Object object) {
    if (!variableSize)
      return;
    array.add(object);
    processModelEvent(new VectorModelEvent(this, VectorModelEvent.ITEM_ADDED, array.size()));
  }

  public void addItem(int aheadOf, Object object) {
    if (!variableSize)
      return;
    array.insert(aheadOf, object);
    processModelEvent(new VectorModelEvent(this, VectorModelEvent.ITEM_ADDED, aheadOf));
  }

  public void remove(int index) {
    if (!variableSize)
      return;
    array.remove(index);
    processModelEvent(new VectorModelEvent(this, VectorModelEvent.ITEM_REMOVED, index));
  }

  public void removeAll() {
    if (!variableSize)
      return;
    if (array.size() > 0) {
      array.clear();
      processModelEvent(new VectorModelEvent(this, VectorModelEvent.STRUCTURE_CHANGED));
    }
  }

  public void enableModelEvents(boolean enable) {
    if (events != enable) {
      events = enable;
      if (enable)
        processModelEvent(new VectorModelEvent(this, VectorModelEvent.STRUCTURE_CHANGED));
    }
  }

  // Vector Model Events

  protected void processModelEvent(VectorModelEvent e) {
    if (events && modelListeners.hasListeners())
      modelListeners.dispatch(e);
  }

  // other jgl.Array methods passed thru

  public ArrayIterator begin() {
    return array.begin();
  }

  public ArrayIterator end() {
    return array.end();
  }

  public Array getArray() {
    return array;
  }

  // Serialization support

  private void writeObject(ObjectOutputStream s) throws IOException {
    s.defaultWriteObject();
    Array newArray = new Array();
    for (int i = 0; i < array.size(); i++) {
      if (array.at(i) instanceof Serializable)
        newArray.add(array.at(i));
    }
    s.writeObject(newArray);
  }

  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
    s.defaultReadObject();
    array = (Array)s.readObject();
  }

  private transient Array array;
  private boolean variableSize = true;
  private transient EventMulticaster modelListeners = new EventMulticaster(); // ignore for serialization
  private boolean events = true;
}
