/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.model;

import borland.jbcl.util.*;
import java.lang.Character;
import java.io.*;

public class IntegerFormatter extends VariantFormatter implements Serializable
{
  public IntegerFormatter(int type) {
    super();
    this.type = type;
  }

  public final String format(Variant value) {
    return (value == null || value.isNull())
              ? ""
              : new Integer(value.getAsInt()).toString();
  }


  public void parse(String stringValue, Variant value) throws InvalidFormatException {
    if (stringValue == null || stringValue.length() == 0) {
      value.setUnassignedNull();
      return;
    }
    setFromInt(value, type, new Integer(stringValue).intValue());
  }

  public void parse(Variant variant, char[] value, int offset, int len) throws InvalidFormatException {
    if (value == null || len == 0 || value.length == 0) {
      variant.setUnassignedNull();
      return;
    }
    boolean negative  = (value[offset] == '-');
    int     digit;
    int     result  = 0;
    if (negative) {
      ++offset;
      --len;
    }
    for (int index = offset; len > 0; ++index, --len) {
      digit = value[index];
      if (digit>= '0' && digit <= '9')
        digit = digit - '0';
      else {
        digit = Character.digit((char)digit,10);
        if (digit < 0)
          throw new InvalidFormatException(new String(value, offset, len));
      }
      result = result * 10 + digit;
    }

    setFromInt(variant, type, negative ? -result : result);
  }

  public int getVariantType() {
    return type;
  }

  protected int type;
}
