/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */

package borland.jbcl.model;

import java.util.*;

import java.awt.*;
import java.awt.Dimension;
import java.awt.Event;

public interface ItemEditMaskChar  //ObjIntf  
{
 //public boolean canChange();  // returns true if this part can be edited
 public boolean isOptional();  // returns true if the char in this position is optional
 public boolean isValid(char c);  // returns true if given char is valid
}

// Base class for all ItemEditMaskCharObj's
class ItemEditMaskCharObj implements ItemEditMaskChar //ObjIntf
{
 protected ItemEditMaskRegion region;
 protected boolean optional;

 public ItemEditMaskCharObj(ItemEditMaskRegion region, boolean optional) {
   this.region = region;
   this.optional = optional;

 }
 public boolean isOptional() {
   return optional;
 }
 public boolean isValid(char c) {
   return true;
 }
}

// ItemEditMaskCharLiteral: This class handles literals within the edit buffer
class ItemEditMaskCharLiteral extends ItemEditMaskCharObj
{
  public ItemEditMaskCharLiteral(ItemEditMaskRegion region) {
    super(region, false);
  }
}


/*
Historic Note: This format is shared by Visual Basic, Access, and Delphi.  That is the reason for
               the choice.  We reserve the right to make extensions to this (supersetting the others)
         as time becomes available.

*/

// Class for '9' (Optional digits 0:9)
class ItemEditMaskCharDigit extends ItemEditMaskCharObj
{
  public ItemEditMaskCharDigit(ItemEditMaskRegion region,  boolean optional) {
    super(region, optional);
  }
  public boolean isValid(char c) {
    return Character.isDigit(c);
  }
}


// Class for '#' (Optional digits, +- allowed, blanks convert to spaces
class ItemEditMaskCharDigitSign extends ItemEditMaskCharDigit
{
  public ItemEditMaskCharDigitSign(ItemEditMaskRegion region,  boolean optional) {
    super(region, optional);                  // digits don't need convesion
  }

  public boolean isValid(char c) {
    if (super.isValid(c))
      return true;
    if (c == '+' || c == '-')
      return true;
    return Character.isWhitespace(c);
  }

}

// Class for '?' (A:Z, optional)
class ItemEditMaskCharAlpha extends ItemEditMaskCharObj
{
  public ItemEditMaskCharAlpha(ItemEditMaskRegion region,  boolean optional) {
    super(region, optional);
  }

  public boolean isValid(char c) {
    return Character.isLetter(c);
  }
}


// Class for 'a' (letter or digit optional)
class ItemEditMaskCharAlphaNum extends ItemEditMaskCharObj
{
  public ItemEditMaskCharAlphaNum(ItemEditMaskRegion region,  boolean optional) {
    super(region, optional);
  }
  public boolean isValid(char c) {
    return Character.isLetterOrDigit(c);
  }
}

// Class for 'C' (char or space optional)
class ItemEditMaskCharCharOrSpace extends ItemEditMaskCharObj
{
  public ItemEditMaskCharCharOrSpace(ItemEditMaskRegion region,  boolean optional) {
    super(region, optional);
  }
  public boolean isValid() {
    return true;        // TODO: find out what is a legal 'char'
  }
}
