/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */

package borland.jbcl.model;

import java.util.*;
import borland.jbcl.util.*;
import java.io.*;

/**
* This class carries "state" information for a control while it is using an ItemEditMask interface.
* The control owns this information, though it is instantiated by the ItemEditMask.  This allows multiple
* controls to share a common ItemEditMask.
*/
public class ItemEditMaskState implements Serializable
{
  private static final long serialVersionUID = 200L;

  public StringBuffer   displayString;    // The buffer which the control should display
            // Note: this buffer must NEVER grow, since we maintain
            // a parallel mapping from char pos to edit region UNLESS we are dealing
            // with a patternless string, in which case it grows on demand
  public int  cursorPos;        // Where the cursor is (input) or should be (output)
  int         variantType;      // The type of variant delivered initially
  transient Object privateObject;    // Optional: any object the EditMasker wants to attach

  public ItemEditMaskState() {
    this(16, 0);
  }
  public ItemEditMaskState(int size, int cursorPos) {
   displayString = new StringBuffer(size);  // the text which the control should show to the user
   this.cursorPos = cursorPos;      // input and output -- it shows where cursor is or should be
   privateObject = null;      // to be used in any way EditMask implementor wants
  }

  // Serialization support

  private void writeObject(ObjectOutputStream s) throws IOException {
    s.defaultWriteObject();
    s.writeObject(privateObject instanceof Serializable ? privateObject : null);
  }

  private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
    s.defaultReadObject();
    privateObject = s.readObject();
  }
}
