/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//------------------------------------------------------------------------------

package borland.jbcl.model;
import borland.jbcl.util.*;
import java.lang.Character;
import java.io.*;

public class LongFormatter extends VariantFormatter implements Serializable
{
  public LongFormatter() {
    super();
  }

  public final String format(Variant value) {
    return (value == null || value.isNull())
              ? ""
              : new Long(value.getLong()).toString();
  }


  public final void parse(String stringValue, Variant value) {
    if (stringValue == null || stringValue.length() == 0) {
      value.setUnassignedNull();
      return;
    }
    value.setLong(new Long(stringValue).longValue());
  }


  public final void parse(Variant variant, char[] value, int offset, int len) throws InvalidFormatException {
    if (value == null || len == 0 || value.length == 0) {
      variant.setUnassignedNull();
      return;
    }
    boolean negative  = (value[offset] == '-');
    int     digit;
    int     result      = 0;
    long    longResult  = 0;
        if (negative) {
      ++offset;
      --len;
    }
    for (int index = offset; len > 0; ++index, --len) {

      digit = value[index];
      if (digit>= '0' && digit <= '9')
        digit = digit - '0';
      else {
        digit = Character.digit((char)digit,10);
        if (digit < 0)
          throw new InvalidFormatException(new String(value, offset, len));
      }
      if (result > 99999999) {
        if (longResult == 0)
          longResult  = result;
        longResult  = longResult * 10 + digit;
      }
      else
        result = result * 10 + digit;
    }

    if (longResult == 0)
      variant.setLong((long)(negative ? -result : result));
    else
      variant.setLong(negative ? -longResult : longResult);
  }

  public int getVariantType() { return Variant.LONG; }
}
