/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
package borland.jbcl.model;

import java.util.*;
import java.lang.reflect.*;
import java.io.InputStream;

/**
  * Loads a ResourceBundle using the System classloader.
  * Necessary for applets that load a ResourceBundle that belongs to the
  * system (e.g. java.text.resources.LocaleElements).
  */

class SystemResourceBundle {
  /**
    * Load the named resource bundle for the given locale.
    */
  static ResourceBundle getBundle(String name, Locale locale) 
                          throws MissingResourceException
  {
    return ResourceLoader.getBundle(null, name, locale);
  }
  /*
   * Load the named resource bundle for the default locale.
   */
  static ResourceBundle getBundle(String name) throws MissingResourceException
  {
    return getBundle(name, Locale.getDefault());
  }
}


/**
  * Provides functionality missing in java.util.ResourceBundle.
  * This class provides an equivalent to java.util.ResourceBundle.getBundle()
  * that allows a specified classloader to be used.  java.util.ResourceBundle()
  * always uses the class loader of its caller.
  */
class ResourceLoader {
  static ResourceBundle getBundle(ClassLoader loader, String name, Locale locale) 
                          throws MissingResourceException
  {
    try {
      if (loader==null) {
        // Trying to load a system class...
        if (ResourceLoader.class.getClassLoader()==null) {
          // ...from a local application.
          return java.util.ResourceBundle.getBundle(name, locale);
        }
        else {
          // ...from an applet.
          // We need to call ResourceBundle.getBundle() from the context
          // of a local class.
          Class clazz = Class.forName("java.util.ResourceBundle");
          Method getBundleMethod = clazz.getMethod("getBundle",
                                      new Class[]{String.class, Locale.class});
          return (ResourceBundle)getBundleMethod.invoke(null, new Object[]{name, locale});
        }
      }
      else {
        // Trying to load a classloader resource...
        if (ResourceLoader.class.getClassLoader()==null) {
          // ...from a local application
          Class clazz = Class.forName("java.util.ResourceBundle");
          Method getBundleMethod = clazz.getMethod("getBundle",
                                      new Class[]{String.class, Locale.class});
          return (ResourceBundle)getBundleMethod.invoke(null, new Object[]{name, locale});
        }
        else {
          // ...from an applet
          Class clazz = loader.loadClass("java.util.ResourceBundle");
          Method getBundleMethod = clazz.getMethod("getBundle",
                                      new Class[]{String.class, Locale.class});
          return (ResourceBundle)getBundleMethod.invoke(null, new Object[]{name, locale});
        }  
      }

    } catch (Exception e) {
      throw new MissingResourceException("can't find resource for ", //NORES
                                            name, "");
    }
    
  }
  static ResourceBundle getBundle(ClassLoader loader, String name) throws MissingResourceException
  {
    return getBundle(loader, name, Locale.getDefault());
  }
}

