/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.util;

import java.util.*;

/**
 * <code>ArrayResourceBundle</code> is a abstract subclass of
 * <code>ResourceBundle</code> that manages locale-dependent resources
 * in an array.  By using numeric references rather than string
 * references, it requires less overhead and provides better performance than 
 * <code>PropertyResourceBundle</code> and <code>ListResourceBundle</code>. 
 * See <code>ResourceBundle</code> for more information about resource
 * bundles in general.
 *
 * <P>
 * Subclasses must override <code>getContents</code> and provide an array,
 * where each item in the array is the resource value.  The key for each
 * resource value is its numeric offset in the array.  For example, the first
 * element in the array has the key 0.  It may be retrieved by
 * using either getObject(0) or getObject("0");
 *
 * <P>
 * Unlike ListResourceBundle and PropertyResourceBundle, where each
 * locale-specific variation of a bundle can override only selected resources,
 * with ArrayResourceBundle, each variation must provide the complete
 * set of Resources.  For example,if MyResources has three resources,
 * then MyResources_ja and MyResources_fr must also have three resources.
 *
 * <p>
 * The following example shows the structure of a ResourceBundle
 * based on ArrayResourceBundle.
 * <pre>
 * class MyResource extends ArrayResourceBundle {
 * 	public Object[] getContents() {
 * 		return contents;
 * 	}
 * 	static final Object[] contents = {
 * 	// LOCALIZE THIS
 * 		"Yes",    // Label for the YES button
 * 		"No",     // Label for the NO button
 * 		"Cancel"  // Label for the CANCEL button
 * 	// END OF MATERIAL TO LOCALIZE
 * 	};
 * }
 * </pre>
 * 
 * </blockquote>
 * @see ResourceBundle
 * @see ListResourceBundle
 * @see PropertyResourceBundle
 */
public abstract class ArrayResourceBundle extends ResourceBundle {

  /**
   * Get an object from an ArrayResourceBundle.
   * <BR>Convenience method to save casting.
   * @param key see class description.
   */
  public final String getString(int key) throws MissingResourceException {
    return (String) getObject(key);
  }

  /**
   * Get an object from a ResourceBundle.
   * <BR>Convenience method to save casting.
   * @param key see class description.
   */
  public final String[] getStringArray(int key)
    throws MissingResourceException {
    return (String[]) getObject(key);
  }

  private Object[] contents;

  /**
   * See class description.
   */
  protected abstract Object[] getContents();

  protected Object handleGetObject(String key) {
    return getObject(Integer.parseInt(key));
  }
  
  /**
   * Get an object from a ResourceBundle.
   * @param key see class description.
   */
  public Object getObject(int index) {
    if (contents == null)
      contents = getContents();
    try {
      return contents[index];
    }
    catch (IndexOutOfBoundsException e) {
      return null;
    }
  }

  /**
   * Return an enumeration of the keys.
   */
  public Enumeration getKeys() {
    if (contents == null)
      contents = getContents();
    Vector result = new Vector(contents.length);
    for (int i=0; i<contents.length; i++) {
      result.addElement(String.valueOf(i));
    }
    return result.elements();
  }
}
