/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.util;

import java.beans.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

public abstract class BasicBeanInfo implements BeanInfo
{
  // Basic Info

  /**
   * The JavaBean component class (this is the only REQUIRED field)
   */
  protected Class beanClass;

  /**
   * The JavaBean's customizer class.
   */
  protected Class customizerClass;

  // Property Info

  /**
   * The property information for the JavaBean. By default will introspect for properties, override to customize.
   *
   * Format:  {{"PropertyName", "Short Description", "ReadMethod", "WriteMethod" [,"PropertyEditor"]}, ...}
   * Example: {{"text", "Control Text", "getText", "setText", null}, ...}
   */
  protected String[][] propertyDescriptors;

  /**
   * The index of the default property for the JavaBean. The default is to not specify.
   */
  protected int defaultPropertyIndex = -1;

  // Method Info

  /**
   * The method names (non-properties) for your JavaBean. By default will introspect for methods, override to customize.
   *
   * Format:  {"method1", "method2", "method3", ...}
   * Example: {"fillRect", "eraseRect", "close", "open"}
   */
  protected String[] methodNames;

  /**
   * The method parameters for each of your JavaBean's methods. Required if methodNames is filled in.
   *
   * Format:  {{"method1Parameter1", "method1Parameter2", ...}, ...}
   * Example: {{"java.awt.Graphics", "java.awt.Rectangle", ...}, ...}
   */
  protected String[][] methodParameters;

  // Event Info

  /**
   * The event information for the JavaBean. By default will introspect for events, override to customize.
   *
   * Format:  {{"EventSetName", "EventListenerClass", "AddMethod", "RemoveMethod"}, ...}
   * Example: {{"ActionListener", "java.awt.event.ActionListener", "addActionListener", "removeActionListener"}, ...}
   */
  protected String[][] eventSetDescriptors;

  /**
   * The names of each event set's listener methods.
   *
   * Format:  {{"listener1Method1", "listener1Method2", "listener1Method3", ...}, ...}
   * Example: {{"actionPerformed"}, ...}
   */
  protected String[][] eventListenerMethods;

  /**
   * The index of the default event for the JavaBean. The default is to not specify.
   */
  protected int defaultEventIndex = -1;

  // Icon Info

  /**
   * A 16x16 color icon for the JavaBean.  If none is specified, one will be searched
   * for using a default resource name.
   */
  protected Image iconColor16x16;

  /**
   * A 32x32 color icon for the JavaBean.  If none is specified, one will be searched
   * for using a default resource name.
   */
  protected Image iconColor32x32;

  /**
   * A 16x16 monochromatic icon for the JavaBean.  If none is specified, one will be
   * searched for using a default resource name.
   */
  protected Image iconMono16x16;

  /**
   * A 32x32 monochromatic icon for the JavaBean.  If none is specified, one will be
   * searched for using a default resource name.
   */
  protected Image iconMono32x32;

  // Additional Info

  /**
   * Any additional BeanInfo for this JavaBean.
   */
  protected BeanInfo[] additionalBeanInfo;

  // Named attributes

  /**
   * Any additional named attributes for the JavaBean.
   *
   * Format:  {{"AttributeName", AttributeSetting}, ...}
   * Example: {{"isContainer", Boolean.TRUE}, {"containerDelegate", "getContentPane"}, ...}
   */
  protected Object[][] namedAttributes;


  // BeanInfo Implementation

  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor bd = new BeanDescriptor(beanClass, customizerClass);
    if (namedAttributes != null && namedAttributes.length > 0) {
      for (int i = 0; i < namedAttributes.length; i++) {
        if (namedAttributes[i].length >= 2 && namedAttributes[i][0] != null)
          bd.setValue(namedAttributes[i][0].toString(), namedAttributes[i][1]);
      }
    }
    return bd;
  }

  public BeanInfo[] getAdditionalBeanInfo() {
    return additionalBeanInfo;
  }

  public PropertyDescriptor[] getPropertyDescriptors() {
    if (propertyDescriptors == null)
      return null;
    try {
      PropertyDescriptor[] pds = new PropertyDescriptor[propertyDescriptors.length];
      ClassLoader cl = getClass().getClassLoader();
      Class editorClass = null;
      for (int i = 0; i < pds.length; i++) {
        pds[i] = new PropertyDescriptor(propertyDescriptors[i][0],
                                        beanClass,
                                        propertyDescriptors[i][2],
                                        propertyDescriptors[i][3]);
        if (propertyDescriptors[i][1] != null)
          pds[i].setShortDescription(propertyDescriptors[i][1]);
        if (propertyDescriptors[i].length > 4 && propertyDescriptors[i][4] != null) {
          try {
            editorClass = Class.forName(propertyDescriptors[i][4]);
          }
          catch (ClassNotFoundException e) {
            editorClass = cl.loadClass(propertyDescriptors[i][4]);
          }
          pds[i].setPropertyEditorClass(editorClass);
        }
      }
      return pds;
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (IntrospectionException e) {
      e.printStackTrace();
    }
    return null;
  }

  public int getDefaultPropertyIndex() {
    return defaultPropertyIndex;
  }

  public MethodDescriptor[] getMethodDescriptors() {
    if (methodNames == null)
      return null;
    try {
      MethodDescriptor[] mds = new MethodDescriptor[methodNames.length];
      ClassLoader cl = getClass().getClassLoader();
      for (int i = 0; i < mds.length; i++) {
        Class[] params = new Class[methodParameters[i].length];
        for (int j = 0; j < params.length; j++) {
          try {
            params[j] = Class.forName(methodParameters[i][j]);
          }
          catch (ClassNotFoundException e) {
            params[j] =  cl.loadClass(methodParameters[i][j]);
          }
        }
        mds[i] = new MethodDescriptor(beanClass.getMethod(methodNames[i], params));
      }
      return mds;
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (NoSuchMethodException e) {
      e.printStackTrace();
    }
    return null;
  }

  public EventSetDescriptor[] getEventSetDescriptors() {
    if (eventSetDescriptors == null)
      return null;
    try {
      EventSetDescriptor[] esds = new EventSetDescriptor[eventSetDescriptors.length];
      Class eventSetClass = null;
      ClassLoader cl = getClass().getClassLoader();
      for (int i = 0; i < esds.length; i++)  {
        try {
          eventSetClass = Class.forName(eventSetDescriptors[i][1]);
        }
        catch (ClassNotFoundException e) {
          eventSetClass =  cl.loadClass(eventSetDescriptors[i][1]);
        }
        esds[i] = new EventSetDescriptor(beanClass,
                                         eventSetDescriptors[i][0],
                                         eventSetClass,
                                         eventListenerMethods[i],
                                         eventSetDescriptors[i][2],
                                         eventSetDescriptors[i][3]);
      }
      return esds;
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (IntrospectionException e) {
      e.printStackTrace();
    }
    return null;
  }

  public int getDefaultEventIndex() {
    return defaultEventIndex;
  }

  /**
   * This method returns the 'default' resource to use to find an icon
   * for a JavaBean.<P>
   *
   * If an icon is requested, and not explicitly set in the subclass of
   * BasicBeanInfo, an icon will be searched for using a simple look-up
   * using the beanClass's name and the requested icon type.  Following
   * is a simple chart describing the default resource locations using
   * an example of the JavaBean class: package1.package2.MyBean<P>
   * <DL>
   * <DT>ICON_COLOR_16x16  <DD>image/MyBean_Color16.gif
   * <DT>ICON_COLOR_32x32  <DD>image/MyBean_Color32.gif
   * <DT>ICON_MONO_16x16   <DD>image/MyBean_Mono16.gif
   * <DT>ICON_MONO_32x32   <DD>image/MyBean_Mono32.gif
   * </DL>
   *
   * @param iconKind the int representing the icon type from java.beans.BeanInfo
   * @return A string representing the appropriate resource name.
   */
  protected String getDefaultIconResource(int iconKind) {
    // convert "package1.package2.MyBean" to "image/MyBean"
    String resource = "image/" + beanClass.getName().substring(beanClass.getName().lastIndexOf('.') + 1); // NORES
    switch (iconKind) {
      case ICON_COLOR_16x16: resource += "_Color16"; break; // NORES
      case ICON_COLOR_32x32: resource += "_Color32"; break; // NORES
      case ICON_MONO_16x16:  resource += "_Mono16"; break; // NORES
      case ICON_MONO_32x32:  resource += "_Mono32"; break; // NORES
    }
    return resource + ".gif";
  }

  public Image getIcon(int iconKind) {
    Image icon = null;
    switch (iconKind) {
      case ICON_COLOR_16x16:
        icon = iconColor16x16;
      case ICON_COLOR_32x32:
        icon = iconColor32x32;
      case ICON_MONO_16x16:
        icon = iconMono16x16;
      case ICON_MONO_32x32:
        icon = iconMono32x32;
    }
    if (icon == null && beanClass != null) {
      String resource = getDefaultIconResource(iconKind);
      icon = getImage(resource);
    }
    return icon;
  }

  /**
   * This is a simple utility function to allow one to retrieve an
   * Image object from a resource.  The resource must by specified as
   * a relative path to the beanClass resource.
   *
   * @param resource the resource name relative to the beanClass resource.
   * @return A java.awt.Image object if the resource can be resolved into one,
   *         otherwise, null.
   */
  public Image getImage(String resource) {
    try {
      if (beanClass != null) {
        URL url = beanClass.getResource(resource);
        return Toolkit.getDefaultToolkit().createImage((ImageProducer)url.getContent());
      }
    }
    catch (Exception x) {
      return null;
    }
    return null;
  }
}
