/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
package borland.jbcl.util;

import java.lang.*;
import java.util.*;
import java.text.*;

/**
 * The BooleanFormat class allows for the creation of a formatter interface
 * via the use of a pattern string.  The format of the pattern is:
 *    "trueString ; falseString { ; nullString}"
 */
public class BooleanFormat extends java.text.Format {
  
  static int TRUESTR  = 0;    // offsets into strings[] for each
  static int FALSESTR = 1;
  static int NULLSTR  = 2;
  
  private String pattern;
  private String[] strings = new String[3];
  
  public BooleanFormat () {
    applyPattern(null);
  }
  
  public BooleanFormat (String pattern) {
    this();
    applyPattern(pattern);
  }
  
  public void applyPattern(String pattern) {
    // If we don't have a pattern or someone has asked for the null
    // pattern, we default to a simple "true;false" pattern
    if (pattern == null || pattern.length() == 0)
      pattern = "true;false";	// NORES
    this.pattern = pattern;
    decomposePattern();
  }
  
  public String toPattern() {
    return (pattern == null) ? null : pattern;
  }
  
  private void decomposePattern() {

    strings[0] = strings[1] = strings[2] = "";
    
    if (pattern == null || pattern.length() == 0)
      return;
    //System.err.println("decomposing pattern: " + pattern);
    
    String workStr = pattern;
    for (int i = 0; i < strings.length; ++i) {
      //System.err.println(" workStr is [" + workStr + "]");
      int index = workStr.indexOf(";");
      //System.err.println(" index is " + index);
      if (index < 0)                // no semi means everything remaining belongs to this string
        index = workStr.length();
      if (index > 0)
        strings[i] = workStr.substring(0, index).trim();
      //System.err.println("  strings[" + i + "] is " + strings[i]);
      if (index >= workStr.length())
        break;
      workStr = workStr.substring(index+1, workStr.length());
    }
/*
 * 
 *    StringTokenizer tokenizer = new StringTokenizer(pattern, ";");
 *    for (int i = 0; i < strings.length && tokenizer.hasMoreTokens(); ++i) {
 *      strings[i] = tokenizer.nextToken();
 *      System.err.println("boolean.decomposePattern[" + i + "] is " + strings[i]);
 *    }
 */
  }
  
  public StringBuffer format(Boolean value, StringBuffer result, FieldPosition pos) {
    //System.err.println("boolean.format(" + value + ")");
    int i = (value == null)
                ? NULLSTR
                : (value.booleanValue())
                  ? TRUESTR
                  : FALSESTR;
    if (result == null)
      result = new StringBuffer();
    result.append(strings[i]);
    //System.err.println(" returning " + result);
    
    return result;
  }
  
  public Boolean parse(String text, ParsePosition pos) {

    //System.err.println("boolean.parse(" + text + ")");

    // Empty string counts as legal "unassigned", even if
    // the nullString is nonblank.
    if (text == null)
      return null;

    int posIndex = pos.getIndex();
    
    // Fetch out the substring starting at the given parsePosition
    String s = text.substring(posIndex);
    int textLen = s.length();

    // Part of the way we differentiate a null meaning unparsable from
    // a null meaning this is the null string is whether the ParsePosition
    // is advanced.  Advance it now presuming success.
    pos.setIndex(posIndex + textLen);
    
    if (textLen == 0)
      return null;

    // Need to strip out the underscore characters and make them spaces
    char[] chars = new char[textLen];
    s.getChars(0, textLen, chars, 0);
    for (int i = 0; i < textLen; ++i)
      if (chars[i] == '_')
        chars[i] = ' ';
    s = new String(chars);
    
    s = s.trim();
    textLen = s.length();
    if (textLen == 0)
      return null;

    //System.err.println(" actual parse string is " + s);

    if (s.equalsIgnoreCase(strings[TRUESTR])) {
      //System.err.println("matched true");
      return new Boolean(true);
    }
    
    if (s.equalsIgnoreCase(strings[FALSESTR])) {
      //System.err.println("matched false");
      return new Boolean(false);
    }

    if (s.equalsIgnoreCase(strings[NULLSTR])) {
      //System.err.println("matched null");
      return null;
    }

    String abbrevTrue = "";
    String abbrevFalse = "";
    String abbrevNull = "";
    if (textLen <= strings[TRUESTR].length()) {
      abbrevTrue = strings[TRUESTR].substring(0, s.length());
      //System.err.println("true abbreviation is: " + abbrevTrue);
    }
    if (textLen <= strings[FALSESTR].length()) {
      abbrevFalse = strings[FALSESTR].substring(0, s.length());
      //System.err.println("false abbreviation is: " + abbrevFalse);
    }
    if (textLen <= strings[NULLSTR].length()) {
      abbrevNull = strings[NULLSTR].substring(0, s.length());
      //System.err.println("null abbreviation is: " + abbrevNull);
    }

    boolean isTrue = s.equalsIgnoreCase(abbrevTrue);
    boolean isFalse = s.equalsIgnoreCase(abbrevFalse);
    boolean isNull = s.equalsIgnoreCase(abbrevNull);

    //System.err.println("abbrev matches are: " + isTrue + ", " + isFalse + ", " + isNull);
    
    if (isTrue && !isFalse && !isNull) {
      //System.err.println("matched true abbrev");
      return new Boolean(true);
    }

    if (isFalse && !isTrue && !isNull) {
      //System.err.println("matched false abbrev");
      return new Boolean(false);
    }

    if (isNull && !isTrue && !isFalse) {
      //System.err.println("matched null abbrev");
      return null;
    }

    // We don't know what this is -- reset the parseposition to tell caller
    // we could not parse it.
    pos.setIndex(posIndex);
    return null;
  }
  
  
  public final Object parseObject(String source, ParsePosition pos) {
    return parse(source, pos);
  }
  
  public final StringBuffer format(Object obj,
                                   StringBuffer toAppendTo,
                                   FieldPosition pos) {
    return format(obj.toString(), toAppendTo, pos);
  }

  public String getTrueString() {
    return strings[TRUESTR];
  }
  public String getFalseString() {
    return strings[FALSESTR];
  }
  public String getNullString() {
    return strings[NULLSTR];
  }
}
