/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
//--------------------------------------------------------------------------------------------------
// Copyright (c) 1996-1998 Borland International, Inc. All Rights Reserved.
//--------------------------------------------------------------------------------------------------

package borland.jbcl.util;

import java.io.*;
//import com.objectspace.jgl.*;
import java.util.Vector;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Rectangle;

/**
 *
 */
public class Diagnostic
{
  public static PrintStream out = System.err;

  /**
   * Common counter variable used as 'line numbers' for debug output messages.
   * EXAMPLE useage:
   * Diagnostic.out.println(++Diagnostic.count+"\tdebug message");
   * or (same output)
   * Diagnostic.printlnc("debug message");
   */
  public static int count = 0;

  /**
   * Enable or disable all output of diagnostic messages to System.err.
   */
  public static void enableOutput(boolean enable) {
    out = enable ? System.err : dummy;
  }

  /**
   * Explicitly set stream for diagnostic messages to be sent to.
   */
  public static void setLogStream(PrintStream log) {
    out = log;
  }

  /**
   * Enable or disable the checking of conditions in precondition() and check()
   */
  public static void enableChecking(boolean enable) {
    checking = enable;
  }

  /**
   * Set the minimum threshold for trace and warning output.
   *         0 is highest level
   *   +maxint is lowest level
   *
   * Setting this level to -1 will effectively turn off traces and warnings.
   * NOTE: this does not effect direct access to 'out', nor does it effect output in the
   *       checking functions. Use enableOutput() for that.
   */
  public static void setTraceLevel(int level) {
    outLevel = level;
  }

  public static int getTraceLevel() {
    return outLevel;
  }

  /**
   * Category based tracing and warning.
   * Pass in a unique String, Class or other object that supports a meaningful toString operation.
   * Then when any call to trace or warning that takes a category
   *   (eg. trace(Object category, String description))
   * is called, the trace is only displayed if an addTraceCategory() call was made with the same
   * category object.
   */
  public static void addTraceCategory(Object category) {
    traceCategories.addElement(category);
  }

  public static void removeTraceCategory(Object category) {
    traceCategories.removeElement(category);
  }

  /**
   * Assert a precondition before a method body run. Use this to describe assumed state and
   * parameter values. A precondition is raised if the given condition is not true.
   *
   * An error here usually indicates a problem with the use of the class.
   */
  public static void precondition(boolean condition, String description) throws DiagnosticPreconditionException {
    if (checking && !condition) {
      out.println("precondition failed: " + description);  //NORES
      throw new DiagnosticPreconditionException(description);
    }
  }

  public static void precondition(boolean condition) throws DiagnosticPreconditionException {
    if (checking && !condition) {
      out.println("precondition failed");  //NORES
      throw new DiagnosticPreconditionException();
    }
  }

  /**
   * Check a condition within a method body. Use this to describe assumed results and state
   * after internal operations. A check is raised if the given condition is not true.
   *
   * An error here usually indicates an internal problem with the class.
   */
  public static void check(boolean condition, String description) throws DiagnosticCheckException {
    if (checking && !condition) {
      out.println("check failed: " + description);  //NORES
      throw new DiagnosticCheckException(description);
    }
  }

  public static void check(boolean condition) throws DiagnosticCheckException {
    if (checking && !condition) {
      out.println("check failed");  //NORES
      throw new DiagnosticCheckException();
    }
  }

  /**
   * Shorthands for causing a check exception if the code reaches an unexpected location
   */
  public static void fail(String description) throws DiagnosticCheckException {
    check(false, description);
  }

  public static void fail() throws DiagnosticCheckException {
    check(false);
  }

  public static void fail(Exception ex) throws DiagnosticCheckException {
    out.println(ex.getMessage());
    ex.printStackTrace(out);
    check(false);
  }
  public static void exit(int code) {
    println("Diagnostic.exit:");
    System.exit(code);
  }

  /**
   * Use to mark places where an exception is needed.
   */
  public static void needException() throws DiagnosticCheckException {
    check(false);
  }

  /**
   * Output a warning if the threashold level is high enough, and a condition is true, and general
   * output is enabled
   */
  public static void warn(int level, boolean condition, String description) {
    if (condition && level <= outLevel)
      out.println("wn[" + level + "] " + indent.substring(0,level*2) + description);  //NORES
  }

  /**
   * Output a warning if the category is enabled, and a condition is true, and general
   * output is enabled
   */
  public static void warn(Object category, boolean condition, String description) {
    if (condition && traceCategories.contains(category))
      out.println("wn[" + category.toString() + "] " + description);  //NORES
  }

  static public void warn(Object category, String description) {
    warn(category, true, description);
  }

  /**
   * Output a warning if the category is enabled, and a condition is true, and general
   * output is enabled
   */
  public static void warn(Object category, int level, boolean condition, String description) {
    if (condition && level <= outLevel && traceCategories.contains(category))
      out.println("wn[" + category.toString() + "." + level +  "] " + description);  //NORES
  }

  /**
   * Output a trace if the threashold level is high enough and general output is enabled
   */
  public static void trace(int level, String description) {
    if (level <= outLevel)
      out.println("[" + level + "]" + indent.substring(0,level*2) + description);
  }

  /**
   * Output a trace if the category is enabled and general output is enabled
   */
  public static void trace(Object category, String description) {
    if (traceCategories.contains(category))
      out.println("[" + category.toString() + "] " + description);
  }

  /**
   * Output a trace if the category is enabled and general output is enabled
   */
  public static void trace(Object category, int level, String description) {
    if (level <= outLevel && traceCategories.contains(category))
      out.println("[" + category.toString() + "." + level + "] " + indent.substring(0,level*2) + description);
  }

  /**
   * Print a diagnostic stack trace of the current thread to the diagnostic out stream
   */
  public static void printStackTrace() {
    new Exception("Diagnostic Stack Trace").printStackTrace(out);  //NORES
  }

  public static void printStackTrace(Throwable ex) {
    ex.printStackTrace(out);
  }

  /**
   * print a message to the diagnostic out stream
   */
  public static void println(String message) {

    out.println(message);
  }

  /**
   * print a message to the diagnostic out stream
   */
  public static void print(String message) {
    out.print(message);
  }

  /**
   * print a message to the diagnostic out stream preceded
   * by a line number (incremented count) and a tab character.
   */
  public static void printlnc(String message) {
    out.println(++count + "\t" + message); // NORES
  }

  /**
   * Flushes the diagnostic out stream buffer
   */
  public static void flush() {
    out.flush();
  }

  /**
   * A useful debugging function to display ItemPainter state bits as text
   */
  public static final String getStateAsText(int state) {
    String stateText = ""; // NORES
    for (int i = 0; i < borland.jbcl.model.ItemPainter.states.length; i++) {
      if ((state & borland.jbcl.model.ItemPainter.states[i]) != 0)
        stateText += "|" + borland.jbcl.model.ItemPainter.stateNames[i]; // NORES
    }
    if ("".equals(stateText)) // NORES
      return "<none>"; // NORES
    return stateText.substring(1, stateText.length());
  }

  /**
   * Paints a color-cycled hashmarked rectangle in the passed bounds.  Used for
   * debugging paint messages.
   */
  public static void debugRect(Graphics g, int x, int y, int width, int height) {
    if (g == null)
      return;
    Rectangle clip = g.getClipBounds();
    if (clip == null)
      return;
    g.setClip(x, y, width, height);
    Color c = g.getColor();
    g.setColor(colorWheel.next());
    g.drawRect(x, y, width - 1, height - 1);
    g.drawRect(x + 1, y + 1, width - 3, height - 3);
    // alternate hash directions
    if (debugRectHashLeft) {
      for (int i = 0; i < x + width + height; i += debugRectInc)
        g.drawLine(x, y + i, x + i, y);
    }
    else {
      for (int i = 0; i < x + width + height; i += debugRectInc)
        g.drawLine(x + width, y + i, x + width - i, y);
    }
    debugRectHashLeft = !debugRectHashLeft;
    if (debugRectInc > 15)
      debugRectInc = 5;
    else
      debugRectInc += 2;
    g.setColor(c);
    if (clip != null)
      g.setClip(clip.x, clip.y, clip.width, clip.height);
  }

  // private members

  private static ColorWheel colorWheel = new ColorWheel(Color.red, ColorWheel.RED_TO_YELLOW, 100);
  private static int debugRectInc = 10;
  private static boolean debugRectHashLeft = true;

  private static final DummyPrintStream dummy = new DummyPrintStream();
  private static final String   indent = "                                  ";

  private static int      outLevel = 0;
  private static boolean  checking = true;
  private static Vector    traceCategories = new Vector();
//  private static OrderedSet     traceCategories = new OrderedSet(); // more overhead, but faster with many items
}
