/*
 * Copyright (c) 1997-1998 Borland International, Inc. All Rights Reserved.
 * 
 * This SOURCE CODE FILE, which has been provided by Borland as part
 * of a Borland product for use ONLY by licensed users of the product,
 * includes CONFIDENTIAL and PROPRIETARY information of Borland.  
 *
 * USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS 
 * OF THE LICENSE STATEMENT AND LIMITED WARRANTY FURNISHED WITH
 * THE PRODUCT.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD BORLAND, ITS RELATED
 * COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST ANY CLAIMS
 * OR LIABILITIES ARISING OUT OF THE USE, REPRODUCTION, OR DISTRIBUTION
 * OF YOUR PROGRAMS, INCLUDING ANY CLAIMS OR LIABILITIES ARISING OUT OF
 * OR RESULTING FROM THE USE, MODIFICATION, OR DISTRIBUTION OF PROGRAMS
 * OR FILES CREATED FROM, BASED ON, AND/OR DERIVED FROM THIS SOURCE
 * CODE FILE.
 */
package borland.jbcl.util;

import java.lang.*;
import java.util.*;
import java.text.*;

public class LocaleUtil {

  /**
   * This method will return the locale identified by a single string
   * with "locale_country_variant" as returned by Locale.toString.  This
   * method may ultimately be replaced by a similar service in the java.util.Locale
   * package.
   *
   * @param localeString A single string containing language, country, and variant, separated by '_'
   *        A null or empty string will yield the default locale.  The country and variant are
   *        optional.
   *
   * @return the Locale for the given language, country, and variant
   */
  public static Locale getLocale(String localeString) {
    if (localeString == null || localeString.length() == 0)
      return Locale.getDefault();

    String[] strings = new String[3];
    strings[0] = strings[1] = strings[2] = "";
    
    StringTokenizer tokenizer = new StringTokenizer(localeString, "_");
    for (int i = 0; i < strings.length && tokenizer.hasMoreTokens(); ++i) {
      strings[i] = tokenizer.nextToken();
      //System.err.println("boolean.decomposePattern[" + i + "] is " + strings[i]);
    }

    return new Locale(strings[0], strings[1], strings[2]);
    
  }
}
